@@ -41,7 +41,7 @@
 
 /*-- Current Version --*/
 #if !defined(lint) && !defined(__LINT__)
-static char RCS_Id[] = "$Id: cmd_go.c,v 1.4 2010/02/25 10:50:47 mwesdorp Exp $";
+static char RCS_Id[] = "$Id: cmd_go.c,v 1.7 2014/03/11 21:49:04 mwesdorp Exp $";
 USE(RCS_Id)
 #endif /* !defined(lint) */
 
@@ -78,6 +78,7 @@ int cmd_go( argc, argv )
 	char             *repeat_batch;         /* Dito. */
 	char             *filter;               /* Dito. */
 	char             *filter_prog;          /* Dito. */
+	char             *nosepline;            /* Dito. */
 	char             *sql;
 	int               sql_len;
 	char              pause_buf[5];         /* Buffer for "hit enter" */
@@ -121,7 +122,7 @@ int cmd_go( argc, argv )
 		fprintf( stderr, "\\go: Unbalanced comment tokens encountered\n" );
 		have_error = True;
 	}
-	else while ((ch = sqsh_getopt( argc, argv, "nfhps:m:x;w:d:t;T:" )) != EOF) 
+	else while ((ch = sqsh_getopt( argc, argv, "nfhps:m:x;w:d:t;T:el" )) != EOF) 
 	{
 		switch (ch) 
 		{
@@ -159,6 +160,14 @@ int cmd_go( argc, argv )
 				}
 				break;
 
+			case 'e' :
+				if (env_put( g_env, "echo", "1", ENV_F_TRAN ) == False)
+				{
+					fprintf( stderr, "\\go: -e: %s\n", sqsh_get_errstr() );
+					have_error = True;
+				}
+				break;
+
 			case 'x' :
 				dsp_flags |= DSP_F_X;
 
@@ -174,6 +183,9 @@ int cmd_go( argc, argv )
 			case 'f' :
 				dsp_flags |= DSP_F_NOFOOTERS;
 				break;
+			case 'l' :
+				dsp_flags |= DSP_F_NOSEPLINE;
+				break;
 			case 'm' :
 				dsp_name = sqsh_optarg;
 				break;
@@ -218,11 +230,13 @@ int cmd_go( argc, argv )
 	if( (argc - sqsh_optind) > 1 || have_error) 
 	{
 	    fprintf( stderr, 
-		"Use: \\go [-d display] [-h] [-f] [-n] [-p] [-m mode] [-s sec]\n"
+		"Use: \\go [-d display] [-e] [-h] [-f] [-l] [-n] [-p] [-m mode] [-s sec]\n"
 		"          [-t [filter]] [-w width] [-x [xgeom]] [-T title] [xacts]\n"
 		"     -d display  When used with -x, send result to named display\n"
+		"     -e          Echo SQL buffer to output\n"
 		"     -h          Suppress headers\n"
 		"     -f          Suppress footers\n"
+		"     -l          Suppress line separators with pretty style output mode\n"
 		"     -n          Do not expand variables\n"
 		"     -p          Report runtime statistics\n"
 		"     -m mode     Switch display mode for result set\n"
@@ -306,14 +320,15 @@ int cmd_go( argc, argv )
 	 * Retrieve any variables that may affect the way in which
 	 * we process or display data.
 	 */
-	env_get( g_env, "headers", &headers );
-	env_get( g_env, "footers", &footers );
-	env_get( g_env, "echo", &echo );
-	env_get( g_env, "expand",  &expand );
-	env_get( g_env, "statistics",  &statistics );
-	env_get( g_env, "clear_on_fail",  &clear_on_fail );
-	env_get( g_env, "batch_pause",  &batch_pause );
-	env_get( g_env, "filter",  &filter );
+	env_get( g_env, "batch_pause",   &batch_pause );
+	env_get( g_env, "clear_on_fail", &clear_on_fail );
+	env_get( g_env, "echo",          &echo );
+	env_get( g_env, "expand",        &expand );
+	env_get( g_env, "filter",        &filter );
+	env_get( g_env, "footers",       &footers );
+	env_get( g_env, "headers",       &headers );
+	env_get( g_env, "nosepline",     &nosepline );
+	env_get( g_env, "statistics",    &statistics );
 
 	/*
 	 * If the user didn't request for statistics via the flag, but
@@ -322,10 +337,12 @@ int cmd_go( argc, argv )
 	if( show_stats == False && (statistics != NULL && *statistics == '1') )
 		show_stats = True;
 
-	if( !(dsp_flags & DSP_F_NOHEADERS) && headers != NULL && *headers == '0' )
+	if( !(dsp_flags & DSP_F_NOHEADERS) && headers   != NULL && *headers   == '0' )
 		dsp_flags |= DSP_F_NOHEADERS;
-	if( !(dsp_flags & DSP_F_NOFOOTERS) && footers != NULL && *footers == '0' )
+	if( !(dsp_flags & DSP_F_NOFOOTERS) && footers   != NULL && *footers   == '0' )
 		dsp_flags |= DSP_F_NOFOOTERS;
+	if( !(dsp_flags & DSP_F_NOSEPLINE) && nosepline != NULL && *nosepline == '1' )
+		dsp_flags |= DSP_F_NOSEPLINE;
 
 
 	/*
