@@ -52,7 +52,7 @@
 
 /*-- Current Version --*/
 #if !defined(lint) && !defined(__LINT__)
-static char RCS_Id[] = "$Id: cmd_input.c,v 1.5 2010/01/28 15:30:37 mwesdorp Exp $";
+static char RCS_Id[] = "$Id: cmd_input.c,v 1.9 2013/07/20 16:18:35 mwesdorp Exp $";
 USE(RCS_Id)
 #endif /* !defined(lint) */
 
@@ -60,6 +60,11 @@ USE(RCS_Id)
 static void    input_sigint_jmp  _ANSI_ARGS(( int, void* ));
 static char*   input_strchr      _ANSI_ARGS(( varbuf_t*, char*, int ));
 static int     input_read        _ANSI_ARGS(( varbuf_t*, int ));
+#if defined(USE_READLINE)
+static int     DynKeywordLoad    _ANSI_ARGS(( void )); /* sqsh-2.1.8 - Feature dynamic keyword load */
+#endif
+/* sqsh-2.3 - Check if we currently are in a C style comment in the SQL buffer */
+static int     csc_buffer        _ANSI_ARGS(( varbuf_t* ));
 
 /*
  * The following macro is used to determine if a line of input is
@@ -71,8 +76,8 @@ static int     input_read        _ANSI_ARGS(( varbuf_t*, int ));
 
 
 /*
- * sg_jmp_buf: The following buffer is used to contain the location 
- *             to which this module will return upon receipt of a 
+ * sg_jmp_buf: The following buffer is used to contain the location
+ *             to which this module will return upon receipt of a
  *             SIGINT. It is only used while waiting on input from the
  *             user.
  */
@@ -110,12 +115,15 @@ int cmd_input()
     char        *newline_go ;         /* Value of $newline_go */
     char        *history_shorthand ;  /* Value of $history_shorthand */
     char        *lineno ;             /* Value of $lineno */
+#if defined(USE_READLINE)
+    char        *keyword_dynamic ;    /* Value of $keyword_dynamic */
+    char        *keyword_refresh ;    /* Value of $keyword_refresh */
+#endif
 
     /*-- Misc --*/
     int          exit_status ;        /* Exit status of sub-command */
     JMP_BUF      old_jmp_buf ;        /* Store the previous jmp_buf */
     int          is_cmd ;             /* True if the current line is cmd */
-    int          no_hist ;            /* True if hist should not be updt. */
     job_id_t     job_id ;             /* Id of job launched or completed */
     char        *defer_file ;         /* Name of file holding user output */
     struct stat  stat_buf ;           /* Check for defer file's existence */
@@ -127,6 +135,11 @@ int cmd_input()
     int         cur_lineno;
     int         interactive;
 
+    /*
+     * sqsh-2.2.0 - Extension on semicolon_hack
+     */
+    char     *semicolon_hack2 ;       /* Value of $semicolon_hack2 */
+    char     *str_remainder   = NULL; /* Remainder of input string after ; */
 
     /*
      * Variables that need to be restored before turning to the
@@ -229,36 +242,67 @@ int cmd_input()
      */
     for (;;)
     {
-        no_hist = False;        /* Save history for this buffer */
-
+#if defined(USE_READLINE)
         /*
-         * Clear out the buffer that will be used to place input read
-         * from the user.
+         * sqsh-2.1.8 - Feature dynamic keyword load
+         * If we are in interactive mode and we have keyword_dynamic enabled
+         * then we want to do a refresh of the keyword list when the database
+         * context is changed, i.e. a "use <database>" command was executed.
          */
-        varbuf_clear( read_buf );
-
-        /*
-         * If an input_file was supplied, or no input string was supplied
-         * then call input_read.  By default, if input_read gets a NULL
-         * input_file, stdin is used.
-         */
-        ret = input_read( read_buf, interactive );
-
-        if (ret <= 0)
+        env_get( g_env,          "keyword_dynamic", &keyword_dynamic );
+        env_get( g_internal_env, "keyword_refresh", &keyword_refresh );
+        if (interactive &&
+            keyword_refresh != NULL && *keyword_refresh != '0' &&
+            keyword_dynamic != NULL && *keyword_dynamic != '0')
         {
-            if (ret == 0)
+            (void) DynKeywordLoad();
+            env_set( g_internal_env, "keyword_refresh", "0" );
+        }
+#endif
+
+        /*
+         * sqsh-2.2.0 - Semicolon_hack2. If str_remainder is not NULL
+         * then we have a leftover from the previous loop were a ;
+         * has been processed. So this part of the input should be
+         * processed next. The else branch executes the pre-sqsh-2.2.0
+         * code path.
+         */
+        if (str_remainder != NULL)
+        {
+            str = str_remainder;
+            str_remainder = NULL;
+        }
+        else
+        {
+            /*
+             * Clear out the buffer that will be used to place input read
+             * from the user.
+             */
+            varbuf_clear( read_buf );
+
+            /*
+             * If an input_file was supplied, or no input string was supplied
+             * then call input_read.  By default, if input_read gets a NULL
+             * input_file, stdin is used.
+             */
+            ret = input_read( read_buf, interactive );
+
+            if (ret <= 0)
             {
-                goto loop_leave;
+                if (ret == 0)
+                {
+                    goto loop_leave;
+                }
+                fprintf( stderr, "input: %s\n", sqsh_get_errstr() );
+                goto loop_abort;
             }
-            fprintf( stderr, "input: %s\n", sqsh_get_errstr() );
-            goto loop_abort;
+
+            /*
+             * Pull contents of read_buf out.
+             */
+            str = varbuf_getstr( read_buf );
         }
 
-        /*
-         * Pull contents of read_buf out.
-         */
-        str = varbuf_getstr( read_buf );
-        
         /*
          * The first thing we need to determine is if the current
          * line contains a sqsh command.  This information will be
@@ -276,7 +320,7 @@ int cmd_input()
          * then we pretend it is a buffer recall.  So, we turn
          * it into a logical call to \buf-append.
          */
-        if (is_cmd == False && interactive && *str == '!' && 
+        if (is_cmd == False && interactive && *str == '!' &&
             !isspace((int)*(str+1)))
          {
 
@@ -316,50 +360,93 @@ int cmd_input()
          * input_strchr(), returns the first ';' in str that will not
          * be contained in double quotes when str is appended to
          * g_sqlbuf.
+         * sqsh-2.2.0 - If semicolon_hack2 is set, it doesn't matter
+         * if the input line is a command or not. We still want to
+         * process possible semicolons. So that makes the if statement
+         * a bit more complex.
          */
-        env_get( g_env, "semicolon_hack", &semicolon_hack );
-        if (semicolon_hack != NULL && *semicolon_hack == '1' && !is_cmd &&
-            strchr( str, ';') != NULL && 
-            (ch = input_strchr( g_sqlbuf, str, ';' )) != NULL)
+        env_get( g_env, "semicolon_hack",  &semicolon_hack );
+        env_get( g_env, "semicolon_hack2", &semicolon_hack2 );
+        if ( (strchr( str, ';') != NULL && (ch = input_strchr( g_sqlbuf, str, ';' )) != NULL) &&
+              ((semicolon_hack != NULL && *semicolon_hack == '1' && !is_cmd) ||
+                 (semicolon_hack2 != NULL && *semicolon_hack2 == '1'))
+           )
         {
-            /*
-             * Copy everything up to the ';' into the current work buffer.
-             */
-            if (ch - str != 0)
-            {
-                varbuf_strncat( g_sqlbuf, str, ch - str );
-                varbuf_charcat( g_sqlbuf, '\n' );
-
-                /*
-                 * We now have an extra line.
-                 */
-                env_set( g_env, "lineno", "+1" );
-            }
-
             /*
              * Look up the name of the command that the user wishes
              * to use when a semicolon is encountered.
              */
             env_get( g_env, "semicolon_cmd", &semicolon_cmd );
-
             if (semicolon_cmd == NULL || *semicolon_cmd == '\0')
-            {
                 varbuf_strcpy( sg_buf, "\\go " );
+            else
+                varbuf_strcpy( sg_buf, semicolon_cmd );
+
+            /*
+             * sqsh-2.2.0 - The hack is going to be even worse and worse.
+             * If semicolon_hack2 is set, then we treat a ; as a
+             * command or batch separator, execute the portion before
+             * the ; as a SQL buffer, or as a sqsh command. The remainder
+             * of the string after the ; is saved for later use.
+             */
+            if (semicolon_hack2 != NULL && *semicolon_hack2 == '1')
+            {
+                /*
+                 * replace the ; with end of line
+                */
+                *ch = '\0';
+                /*
+                 * save the remainder of the string in str_remainder
+                 * or set it to NULL if there is really nothing left
+                */
+                str_remainder = ch + 1;
+                if (*str_remainder == '\n' || *str_remainder == '\0')
+                    str_remainder = NULL;
+                /*
+                 * Check if we have to deal with a sqsh command or
+                 * a SQL statement, at least the last part of it.
+                 */
+                if (!is_cmd && jobset_is_cmd( g_jobset, str ) == False)
+                {
+                    if (ch - str != 0)
+                    {
+                        varbuf_strncat( g_sqlbuf, str, ch - str );
+                        varbuf_charcat( g_sqlbuf, '\n' );
+                        env_set( g_env, "lineno", "+1" );
+                    }
+                    str = varbuf_getstr( sg_buf );
+                }
+                /* else the current str is a sqsh command */
             }
             else
             {
-                varbuf_strcpy( sg_buf, semicolon_cmd );
+                /*
+                 * Original pre-sqsh-2.2.0 code path with only
+                 * semicolon_hack set to true.
+                 */
+                /*
+                 * Copy everything up to the ';' into the current work buffer.
+                 */
+                if (ch - str != 0)
+                {
+                    varbuf_strncat( g_sqlbuf, str, ch - str );
+                    varbuf_charcat( g_sqlbuf, '\n' );
+
+                    /*
+                     * We now have an extra line.
+                     */
+                    env_set( g_env, "lineno", "+1" );
+                }
+
+                /*
+                 * Now, stick the semicolon command in the front of everything
+                 * following the semicolon. and turn that into the command
+                 * line.
+                 */
                 varbuf_charcat( sg_buf, ' ' );
+                varbuf_strcat( sg_buf, ch + 1 );
+                str = varbuf_getstr( sg_buf );
             }
-
-            /*
-             * Now, stick the semicolon command in the front of everything
-             * following the semicolon. and turn that into the command
-             * line.
-             */
-            varbuf_strcat( sg_buf, ch + 1 );
-
-            str = varbuf_getstr( sg_buf );
             is_cmd = True;
         }
 
@@ -449,7 +536,7 @@ int cmd_input()
                              * if $interactive is set to 0, then this entry will
                              * automatically be thrown away.
                              */
-                            if (!no_hist && interactive)
+                            if (interactive)
                             {
                                 history_append( g_history, varbuf_getstr(g_sqlbuf) );
 
@@ -463,8 +550,8 @@ int cmd_input()
 
                         case CMD_CLEARBUF:
                             /*
-			     * sqsh-2.1.7 - The same as CMD_RESETBUF but without
-			     * saving the buffer to the history.
+                             * sqsh-2.1.7 - The same as CMD_RESETBUF but without
+                             * saving the buffer to the history.
                              */
                             varbuf_clear( g_sqlbuf );
                             env_set( g_env, "lineno", "=1" ) ;  /* Set to 1 */
@@ -505,7 +592,6 @@ int cmd_input()
                         case CMD_ABORT :
                             goto loop_abort;
                             break;
-                        
 
                         default :
                             sprintf( number, "=%d", cur_lineno );
@@ -521,7 +607,7 @@ int cmd_input()
                  * jobset_run() returned a non-negative value, so it launched
                  * a background process.  The only thing we need to do it
                  * let the user know it was launched.
-		 * sqsh-2.1.7 - Also save and clear the command buffer.
+                 * sqsh-2.1.7 - Also save and clear the command buffer.
                  */
                 default :
                     if (interactive)
@@ -529,14 +615,11 @@ int cmd_input()
                         job_pid = jobset_get_pid( g_jobset, job_id );
                         fprintf( stdout, "Job #%d running [%d]\n", (int)job_id,
                                     (int)job_pid );
-                        if (!no_hist)
-                        {
-                            history_append( g_history, varbuf_getstr(g_sqlbuf) );
+                        history_append( g_history, varbuf_getstr(g_sqlbuf) );
 
-                            /*-- Set histnum to be current history number --*/
-                            sprintf( number, "%d", history_get_nbr(g_history) );
-                            env_set( g_env, "histnum", number );
-                        }
+                        /*-- Set histnum to be current history number --*/
+                        sprintf( number, "%d", history_get_nbr(g_history) );
+                        env_set( g_env, "histnum", number );
                         varbuf_clear( g_sqlbuf );
                         env_set( g_env, "lineno", "=1" ) ;  /* Set to 1 */
                     }
@@ -551,7 +634,7 @@ int cmd_input()
          * than each time the user hits return.
          */
         job_id = 0;
-        while(interactive && 
+        while(interactive &&
             (job_id = jobset_wait(g_jobset, -1, &exit_status, JOB_NONBLOCK)) > 0)
         {
             /*
@@ -649,7 +732,7 @@ loop_done :
 
     if (read_buf != NULL)
         varbuf_destroy( read_buf );
-    
+
     /*
      * Restore the line number to its previous value.
      */
@@ -667,7 +750,7 @@ loop_done :
         varbuf_destroy( g_sqlbuf );
         g_sqlbuf = (varbuf_t*)orig_sqlbuf;
     }
-    
+
     /*
      * Restore the original signal context, and, just in case
      * cmd_loop() has been recursively called, restore the original
@@ -700,7 +783,7 @@ static int input_read( output_buf, interactive )
     char            *exp_prompt   = NULL;
 
     /*
-     * If we are in interactive mode then we need to display a 
+     * If we are in interactive mode then we need to display a
      * prompt to the user.
      */
     if (interactive)
@@ -708,7 +791,7 @@ static int input_read( output_buf, interactive )
         /*
          * If we haven't already allocated a buffer in which to
          * expand the prompt then we should do so.
-	 * sqsh-2.1.6 - expand buffer from 32 to 64 bytes.
+         * sqsh-2.1.6 - expand buffer from 32 to 64 bytes.
          */
         if (sg_prompt_buf == NULL)
         {
@@ -736,13 +819,13 @@ static int input_read( output_buf, interactive )
             if (!is_continued)
             {
                 env_get( g_env, "prompt", &prompt );
-                if( prompt == NULL || *prompt == '\0' ) 
+                if( prompt == NULL || *prompt == '\0' )
                     prompt = "${lineno}> ";
             }
             else
             {
                 env_get( g_env, "prompt2", &prompt );
-                if( prompt == NULL || *prompt == '\0' ) 
+                if( prompt == NULL || *prompt == '\0' )
                     prompt = "--> ";
             }
 
@@ -755,7 +838,7 @@ static int input_read( output_buf, interactive )
                 fprintf( stderr, "prompt: %s\n", sqsh_get_errstr() );
                 varbuf_strcpy( sg_prompt_buf, "?> " );
             }
-            
+
             exp_prompt = varbuf_getstr(sg_prompt_buf);
         }
         else
@@ -790,8 +873,12 @@ static int input_read( output_buf, interactive )
              * table name, [_0-9A-Za-z].  If we hit a comment, then we
              * simply ignore this line, without even incrementing the
              * line number.
+             * sqsh-2.3 - Only ignore sqsh # comments if the current
+             * SQL buffer did not start a C style comment construct,
+             * that is not closed so far, and if we are not inside a
+             * single or double quotes string.
              */
-            if (IS_COMMENT(str))
+            if (IS_COMMENT(str) && csc_buffer( g_sqlbuf ) == 0)
             {
                 if (is_continued)
                 {
@@ -873,13 +960,13 @@ static char* input_strchr( varbuf, str, c )
             case QUOTE_NONE:
                 switch (*cptr)
                 {
-                    case '\'': 
+                    case '\'':
                         quote_type = QUOTE_SINGLE;
                         break;
-                    case '\"': 
+                    case '\"':
                         quote_type = QUOTE_DOUBLE;
                         break;
-                    case '/' : 
+                    case '/' :
                         if (*(cptr + 1) == '*')
                             quote_type = QUOTE_COMMENT;
                         break;
@@ -897,28 +984,28 @@ static char* input_strchr( varbuf, str, c )
                         break;
                 }
                 break;
-            
+
             case QUOTE_COMMENT:
                 if (*cptr == '*' && *(cptr + 1) == '/')
                 {
                     quote_type = QUOTE_NONE;
                 }
                 break;
-            
+
             case QUOTE_SINGLE:
                 if (*cptr == '\'')
                 {
                     quote_type = QUOTE_NONE;
                 }
                 break;
-            
+
             case QUOTE_DOUBLE:
                 if (*cptr == '\"')
                 {
                     quote_type = QUOTE_NONE;
                 }
                 break;
-            
+
             default:
                 break;
         }
@@ -935,13 +1022,13 @@ static char* input_strchr( varbuf, str, c )
             case QUOTE_NONE:
                 switch (*cptr)
                 {
-                    case '\'': 
+                    case '\'':
                         quote_type = QUOTE_SINGLE;
                         break;
-                    case '\"': 
+                    case '\"':
                         quote_type = QUOTE_DOUBLE;
                         break;
-                    case '/' : 
+                    case '/' :
                         if (*(cptr + 1) == '*')
                             quote_type = QUOTE_COMMENT;
                         break;
@@ -959,28 +1046,28 @@ static char* input_strchr( varbuf, str, c )
                         break;
                 }
                 break;
-            
+
             case QUOTE_COMMENT:
                 if (*cptr == '*' && *(cptr + 1) == '/')
                 {
                     quote_type = QUOTE_NONE;
                 }
                 break;
-            
+
             case QUOTE_SINGLE:
                 if (*cptr == '\'')
                 {
                     quote_type = QUOTE_NONE;
                 }
                 break;
-            
+
             case QUOTE_DOUBLE:
                 if (*cptr == '\"')
                 {
                     quote_type = QUOTE_NONE;
                 }
                 break;
-            
+
             default:
                 break;
         }
@@ -991,6 +1078,120 @@ static char* input_strchr( varbuf, str, c )
     return NULL;
 }
 
+/*
+ * csc_buffer():
+ *
+ * sqsh-2.3 - Blast through the SQL buffer that is created up
+ *            till now and see if we are in a quoted or double
+ *            quoted string or in a C style comment.
+ *            Return the value of the quote_type that reflects the
+ *            current situation. (0 means not in quotes or comments)
+ */
+static int csc_buffer( varbuf )
+    varbuf_t   *varbuf;
+{
+#define   QUOTE_NONE            0
+#define   QUOTE_SINGLE          1
+#define   QUOTE_DOUBLE          2
+#define   QUOTE_COMMENT         3
+
+    char   *cptr;
+    int     quote_type      = QUOTE_NONE;
+    int     csclevel        = 0;
+
+
+    if ( (cptr = varbuf_getstr(varbuf)) == NULL )
+        return QUOTE_NONE;
+
+    /*-- Blast through varbuf --*/
+    for (; *cptr != '\0'; ++cptr)
+    {
+        /*
+         * First step over any escape characters and the character
+         * that is escaped by \\ itself, but still do a sanity
+         * check on end of string altogether.
+         */
+        if (*cptr == '\\' && *(cptr + 1) == '\\')
+        {
+            cptr += 3;
+            if (*cptr == '\0' || *(cptr - 1) == '\0')
+                 break;
+        }
+
+        switch (quote_type)
+        {
+            case QUOTE_NONE:
+                switch (*cptr)
+                {
+                    case '\'':
+                        quote_type = QUOTE_SINGLE;
+                        break;
+                    case '\"':
+                        quote_type = QUOTE_DOUBLE;
+                        break;
+                    case '/' :
+                        if (*(cptr + 1) == '*')
+                        {
+                            quote_type = QUOTE_COMMENT;
+                            ++csclevel;
+                            ++cptr;
+                        }
+                        break;
+                    case '-': /* -- comment till end of end line */
+                        if (*(cptr + 1) == '-')
+                        {
+                            while (*cptr != '\n' && *cptr != '\0')
+                                ++cptr;
+
+                            if (*cptr == '\0')
+                                --cptr;
+                        }
+                        break;
+                    default:
+                        break;
+                }
+                break;
+
+            case QUOTE_COMMENT:
+                /*
+                 * C style comments in SQL may be nested, but do not
+                 * take quoting or -- comments in account anymore.
+                 * This is according to isql behavior.
+                 */
+                if (*cptr == '/' && *(cptr + 1) == '*')
+                {
+                    ++cptr;
+                    ++csclevel;
+                }
+                else if (*cptr == '*' && *(cptr + 1) == '/')
+                {
+                    ++cptr;
+                    if (--csclevel == 0)
+                        quote_type = QUOTE_NONE;
+                }
+                break;
+
+            case QUOTE_SINGLE:
+                if (*cptr == '\'')
+                {
+                    quote_type = QUOTE_NONE;
+                }
+                break;
+
+            case QUOTE_DOUBLE:
+                if (*cptr == '\"')
+                {
+                    quote_type = QUOTE_NONE;
+                }
+                break;
+
+            default:
+                break;
+        }
+    }
+    return quote_type;
+}
+
 /*
  * input_sigint_jmp():
  *
@@ -1003,3 +1204,140 @@ static void input_sigint_jmp( sig, user_data )
 {
     LONGJMP( sg_jmp_buf, 1 );
 }
+
+#if defined(USE_READLINE)
+/*
+ * Function: DynKeywordLoad()
+ *
+ * sqsh-2.1.8 - Dynamically execute a query provided by the variable keyword_query
+ * and load the result set into the readline autocompletion list.
+ * By default the query is "select name from sysobjects order by name"
+ * But of course you can change this to anything you like as long as the result set
+ * contains a first column with character data. The variable keyword_query can be
+ * defined in your .sqshrc file for example or in a sqsh_session file.
+ *
+ */
+static int DynKeywordLoad ()
+{
+    CS_COMMAND *cmd;
+    CS_CHAR    *keyword_query;
+    CS_DATAFMT  columns[1];
+    CS_RETCODE  ret;
+    CS_RETCODE  results_ret;
+    CS_INT      result_type;
+    CS_INT      count;
+    CS_INT      idx;
+    CS_INT      datalength[1];
+    CS_SMALLINT indicator [1];
+    CS_CHAR     name   [256];
+
+
+    env_get( g_env, "keyword_query", &keyword_query );
+    if ( keyword_query == NULL || *keyword_query == '\0' ) {
+        DBG(sqsh_debug(DEBUG_ERROR, "DynKeywordLoad: Variable keyword_query is empty.\n"));
+        return (CS_FAIL);
+    }
+    if ( g_connection == NULL )
+    {
+        DBG(sqsh_debug(DEBUG_ERROR, "DynKeywordLoad: g_connection is not initialized.\n"));
+        return (CS_FAIL);
+    }
+    if (ct_cmd_alloc( g_connection, &cmd ) != CS_SUCCEED)
+    {
+        DBG(sqsh_debug(DEBUG_ERROR, "DynKeywordLoad: Call to ct_cmd_alloc failed.\n"));
+        return (CS_FAIL);
+    }
+    if (ct_command( cmd,                /* Command Structure */
+                    CS_LANG_CMD,        /* Command Type      */
+                    keyword_query,      /* Buffer            */
+                    CS_NULLTERM,        /* Buffer Length     */
+                    CS_UNUSED           /* Options           */
+                  ) != CS_SUCCEED)
+    {
+        ct_cmd_drop( cmd );
+        DBG(sqsh_debug(DEBUG_ERROR, "DynKeywordLoad: Call to ct_command failed.\n"));
+        return (CS_FAIL);
+    }
+    if (ct_send( cmd ) != CS_SUCCEED)
+    {
+        ct_cmd_drop( cmd );
+        DBG(sqsh_debug(DEBUG_ERROR, "DynKeywordLoad: Call to ct_send failed.\n"));
+        return (CS_FAIL);
+    }
+
+    (void) sqsh_readline_clear();       /* Empty the current keyword list     */
+    (void) sqsh_readline_load ();       /* Reload keywords from $keyword_file */
+
+    while ((results_ret = ct_results(cmd, &result_type)) == CS_SUCCEED)
+    {
+        switch ((int) result_type)
+        {
+            case CS_ROW_RESULT:
+                columns[0].datatype  = CS_CHAR_TYPE;
+                columns[0].format    = CS_FMT_NULLTERM;
+                columns[0].maxlength = 255;
+                columns[0].count     = 1;
+                columns[0].locale    = NULL;
+                ct_bind(cmd, 1, &columns[0], name, &datalength[0], &indicator[0]);
+
+                while ( ct_fetch (cmd, CS_UNUSED, CS_UNUSED, CS_UNUSED, &count) == CS_SUCCEED )
+                {
+                    /* Remove trailing blanks, tabs and newlines, just in case */
+                    for ( idx = strlen(name) - 1;
+                          idx >= 0 && (name[idx] == ' ' || name[idx] == '\t' || name[idx] == '\n');
+                          name[idx--] = '\0');
+                    sqsh_readline_add (name);    /* Add name to readline linked list of keywords */
+                }
+                break;
+
+            case CS_COMPUTE_RESULT:
+            case CS_CURSOR_RESULT:
+            case CS_MSG_RESULT:
+            case CS_PARAM_RESULT:
+            case CS_STATUS_RESULT:
+                /*
+                 * Just ignore these kind of results.
+                 */
+                while ( ct_fetch (cmd, CS_UNUSED, CS_UNUSED, CS_UNUSED, &count) == CS_SUCCEED );
+                break;
+
+            case CS_CMD_SUCCEED:
+                DBG(sqsh_debug(DEBUG_ERROR, "DynKeywordLoad: No rows returned from query.\n"));
+                ret = CS_FAIL;
+                break;
+
+            case CS_CMD_FAIL:
+                DBG(sqsh_debug(DEBUG_ERROR, "DynKeywordLoad: Error encountered during query processing.\n"));
+                ret = CS_FAIL;
+                break;
+
+            case CS_CMD_DONE:
+                break;
+
+            default:
+                DBG(sqsh_debug(DEBUG_ERROR, "DynKeywordLoad: Unexpected error encountered. (1)\n"));
+                ret = CS_FAIL;
+                break;
+        }
+    }
+
+    switch ((int) results_ret)
+    {
+        case CS_END_RESULTS:
+            ret = CS_SUCCEED;
+            break;
+
+        case CS_FAIL:
+            DBG(sqsh_debug(DEBUG_ERROR, "DynKeywordLoad: Unexpected error encountered. (2)\n"));
+            ret = CS_FAIL;
+            break;
+
+        default:
+            DBG(sqsh_debug(DEBUG_ERROR, "DynKeywordLoad: Unexpected error encountered. (3)\n"));
+            ret = CS_FAIL;
+            break;
+    }
+    ct_cmd_drop( cmd );
+    return ( ret );
+}
+#endif
