@@ -31,7 +31,7 @@
 
 /*-- Current Version --*/
 #if !defined(lint) && !defined(__LINT__)
-static char RCS_Id[] = "$Id: dsp.c,v 1.3 2008/05/21 17:51:24 mpeppler Exp $";
+static char RCS_Id[] = "$Id: dsp.c,v 1.6 2014/01/18 18:36:34 mwesdorp Exp $";
 USE(RCS_Id)
 #endif /* !defined(lint) */
 
@@ -50,7 +50,7 @@ int         g_dsp_interrupted  = False;
 static CS_COMMAND *sg_cmd = NULL;
 
 /*
- * g_dsp_props: This data structure contains the current set of 
+ * g_dsp_props: This data structure contains the current set of
  *      properties defined for the display sub-system. Be careful
  *      when editing this data strcuture.
  */
@@ -178,6 +178,9 @@ int dsp_cmd( output, cmd, sql, flags )
 	sig_install( SIGINT, dsp_signal, (void*)NULL, 0 );
 	sig_install( SIGPIPE, dsp_signal, (void*)NULL, 0 );
 
+	/* sqsh-2.5 - Feature p2f, reset g_p2fc before a new batch is started */
+	g_p2fc = 0;
+
 	if (ct_send( cmd ) != CS_SUCCEED)
 		ret = DSP_FAIL;
 
@@ -188,7 +191,7 @@ int dsp_cmd( output, cmd, sql, flags )
 	 * All that is left is to process the results of the current
 	 * query, reinstall the old signal handlers, and return!
 	 */
-	if (ret == DSP_SUCCEED) 
+	if (ret == DSP_SUCCEED)
 	{
 		if ((flags & DSP_F_NOTHING) != 0)
 		{
@@ -305,12 +308,26 @@ static int dsp_prop_set( prop, ptr, len )
 {
 	switch (prop)
 	{
+		/*
+		 * sqsh-2.1.9 - Introduced and remapped property types DSP_DATETIMEFMT, DSP_DATEFMT
+		 * and DSP_TIMEFMT to implement date and time datatype conversions with strftime as
+		 * already existed for the datetime and smalldatetime datatypes.
+		 * This feature request was filed as bugreport 3603409 on Sourceforge.
+		 */
+		case DSP_DATETIMEFMT:
+			return dsp_datetimefmt_set( (char*)ptr );
+			break;
+
 		case DSP_DATEFMT:
 			return dsp_datefmt_set( (char*)ptr );
 			break;
 
+		case DSP_TIMEFMT:
+			return dsp_timefmt_set( (char*)ptr );
+			break;
+
 		case DSP_COLWIDTH:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
 				"dsp_prop: dsp_prop(DSP_SET, DSP_COLWDTH, %d)\n", *((int*)ptr));)
 
 			if (*((int*)ptr) < 1)
@@ -323,7 +340,7 @@ static int dsp_prop_set( prop, ptr, len )
 			break;
 		
 		case DSP_FLOAT_PREC:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
 				"dsp_prop: dsp_prop(DSP_SET, DSP_FLOAT_PREC, %d)\n", *((int*)ptr));)
 
 			if (*((int*)ptr) < 0 || *((int*)ptr) < g_dsp_props.p_flt_scale)
@@ -336,7 +353,7 @@ static int dsp_prop_set( prop, ptr, len )
 			break;
 
 		case DSP_FLOAT_SCALE:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
 				"dsp_prop: dsp_prop(DSP_SET, DSP_FLOAT_SCALE, %d)\n",
 				*((int*)ptr));)
 
@@ -350,7 +367,7 @@ static int dsp_prop_set( prop, ptr, len )
 			break;
 
 		case DSP_REAL_PREC:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
 				"dsp_prop: dsp_prop(DSP_SET, DSP_REAL_PREC, %d)\n", *((int*)ptr));)
 
 			if (*((int*)ptr) < 0 || *((int*)ptr) < g_dsp_props.p_real_scale)
@@ -363,7 +380,7 @@ static int dsp_prop_set( prop, ptr, len )
 			break;
 
 		case DSP_REAL_SCALE:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
 				"dsp_prop: dsp_prop(DSP_SET, DSP_REAL_SCALE, %d)\n",
 				*((int*)ptr));)
 
@@ -377,7 +394,7 @@ static int dsp_prop_set( prop, ptr, len )
 			break;
 
 		case DSP_STYLE:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
 				"dsp_prop: dsp_prop(DSP_SET, DSP_STYLE, %d)\n", *((int*)ptr));)
 
 			if (!(DSP_VALID_STYLE( *((int*)ptr) )))
@@ -389,8 +406,8 @@ static int dsp_prop_set( prop, ptr, len )
 			g_dsp_props.p_style = *((int*)ptr);
 			break;
 
-		case DSP_WIDTH: 
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
+		case DSP_WIDTH:
+			DBG(sqsh_debug(DEBUG_DISPLAY,
 				"dsp_prop: dsp_prop(DSP_SET, DSP_WIDTH, %d)\n", *((int*)ptr));)
 
 			if (*((int*)ptr) >= 30)
@@ -404,8 +421,8 @@ static int dsp_prop_set( prop, ptr, len )
 			}
 			break;
 
-		case DSP_OUTPUTPARMS: 
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
+		case DSP_OUTPUTPARMS:
+			DBG(sqsh_debug(DEBUG_DISPLAY,
 				"dsp_prop: dsp_prop(DSP_SET, DSP_OUTPUTPARMS, %d)\n",
 				*((int*)ptr));)
 
@@ -421,10 +438,13 @@ static int dsp_prop_set( prop, ptr, len )
 			break;
 		
 		case DSP_COLSEP:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_SET, DSP_COLSEP, '%s')\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_SET, DSP_COLSEP, '%s')\n",
 				(ptr == NULL)?"NULL":((char*)ptr));)
 
+			if (ptr == NULL || strcasecmp (ptr, "default") == 0)
+				ptr = " ";        /* sqsh-2.5 - Set to default when ptr is NULL */
+
 			if (len == DSP_NULLTERM)
 			{
 				len = strlen( (char*)ptr );
@@ -432,7 +452,7 @@ static int dsp_prop_set( prop, ptr, len )
 
 			if (len > MAX_SEPLEN || len < 0)
 			{
-				sqsh_set_error( SQSH_E_INVAL, 
+				sqsh_set_error( SQSH_E_INVAL,
 					"Invalid length of column separator (between 0 and %d allowed)",
 					MAX_SEPLEN );
 				return DSP_FAIL;
@@ -446,10 +466,13 @@ static int dsp_prop_set( prop, ptr, len )
 			break;
 
 		case DSP_BCP_COLSEP:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_SET, DSP_BCP_COLSEP, '%s')\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_SET, DSP_BCP_COLSEP, '%s')\n",
 				(ptr == NULL)?"NULL":((char*)ptr));)
 
+			if (ptr == NULL || strcasecmp (ptr, "default") == 0)
+				ptr = "|";        /* sqsh-2.5 - Set to default when ptr is NULL */
+
 			if (len == DSP_NULLTERM)
 			{
 				len = strlen( (char*)ptr );
@@ -457,7 +480,7 @@ static int dsp_prop_set( prop, ptr, len )
 
 			if (len > MAX_SEPLEN || len < 0)
 			{
-				sqsh_set_error( SQSH_E_INVAL, 
+				sqsh_set_error( SQSH_E_INVAL,
 					"Invalid length of bcp col separator (between 0 and %d allowed)",
 					MAX_SEPLEN );
 				return DSP_FAIL;
@@ -471,10 +494,13 @@ static int dsp_prop_set( prop, ptr, len )
 			break;
 
 		case DSP_BCP_ROWSEP:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_SET, DSP_BCP_ROWSEP, '%s')\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_SET, DSP_BCP_ROWSEP, '%s')\n",
 				(ptr == NULL)?"NULL":((char*)ptr));)
 
+			if (ptr == NULL || strcasecmp (ptr, "default") == 0)
+				ptr = "|";        /* sqsh-2.5 - Set to default when ptr is NULL */
+
 			if (len == DSP_NULLTERM)
 			{
 				len = strlen( (char*)ptr );
@@ -482,7 +508,7 @@ static int dsp_prop_set( prop, ptr, len )
 
 			if (len > MAX_SEPLEN || len < 0)
 			{
-				sqsh_set_error( SQSH_E_INVAL, 
+				sqsh_set_error( SQSH_E_INVAL,
 					"Invalid length of bcp row separator (between 0 and %d allowed)",
 					MAX_SEPLEN );
 				return DSP_FAIL;
@@ -496,8 +522,8 @@ static int dsp_prop_set( prop, ptr, len )
 			break;
 
 		case DSP_BCP_TRIM:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_SET, DSP_BCP_TRIM, '%d')\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_SET, DSP_BCP_TRIM, '%d')\n",
 				(ptr == NULL)?-1:*((int*)ptr));)
 
 			if (*((int*)ptr) == 0)
@@ -511,10 +537,13 @@ static int dsp_prop_set( prop, ptr, len )
 			break;
 		
 		case DSP_LINESEP:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_SET, DSP_LINESEP, '%s')\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_SET, DSP_LINESEP, '%s')\n",
 				(ptr == NULL)?"NULL":((char*)ptr));)
 
+			if (ptr == NULL || strcasecmp (ptr, "default") == 0)
+				ptr = "\n\t";        /* sqsh-2.5 - Set to default when ptr is NULL */
+
 			if (len == DSP_NULLTERM)
 			{
 				len = strlen( (char*)ptr );
@@ -522,7 +551,7 @@ static int dsp_prop_set( prop, ptr, len )
 
 			if (len > MAX_SEPLEN || len < 0)
 			{
-				sqsh_set_error( SQSH_E_INVAL, 
+				sqsh_set_error( SQSH_E_INVAL,
 					"Invalid length of line separator (between 0 and %d allowed)",
 					MAX_SEPLEN );
 				return DSP_FAIL;
@@ -535,8 +564,8 @@ static int dsp_prop_set( prop, ptr, len )
 			break;
 
 		case DSP_XGEOM:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_SET, DSP_XGEOM, '%s')\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_SET, DSP_XGEOM, '%s')\n",
 				(ptr == NULL)?"NULL":((char*)ptr));)
 
 			if (len == DSP_NULLTERM)
@@ -546,7 +575,7 @@ static int dsp_prop_set( prop, ptr, len )
 
 			if (len > MAX_XGEOM || len < 0)
 			{
-				sqsh_set_error( SQSH_E_INVAL, 
+				sqsh_set_error( SQSH_E_INVAL,
 					"Invalid length of X geometry (between 0 and %d allowed)",
 					MAX_XGEOM );
 				return DSP_FAIL;
@@ -557,7 +586,7 @@ static int dsp_prop_set( prop, ptr, len )
 			break;
 
 		case DSP_MAXLEN:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
 				"dsp_prop: dsp_prop(DSP_SET, DSP_MAXLEN, %d)\n", *((int*)ptr));)
 
 			if (*((int*)ptr) < 0)
@@ -591,49 +620,69 @@ static int dsp_prop_get( prop, ptr, len )
 {
 	switch (prop)
 	{
+		/*
+		 * sqsh-2.1.9 - Introduced and remapped property types DSP_DATETIMEFMT, DSP_DATEFMT
+		 * and DSP_TIMEFMT to implement date and time datatype conversions with strftime as
+		 * already existed for the datetime and smalldatetime datatypes.
+		 * This feature request was filed as bugreport 3603409 on Sourceforge.
+		 */
 		case DSP_COLWIDTH:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_GET, DSP_COLWIDTH) = %d\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_GET, DSP_COLWIDTH) = %d\n",
 			   g_dsp_props.p_colwidth);)
 
 			*((int*)ptr) = g_dsp_props.p_colwidth;
 			break;
 
 		case DSP_FLOAT_PREC:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_GET, DSP_FLOAT_PREC) = %d\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_GET, DSP_FLOAT_PREC) = %d\n",
 			   g_dsp_props.p_flt_prec);)
 
 			*((int*)ptr) = g_dsp_props.p_flt_prec;
 			break;
 
 		case DSP_FLOAT_SCALE:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_GET, DSP_FLOAT_SCALE) = %d\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_GET, DSP_FLOAT_SCALE) = %d\n",
 			   g_dsp_props.p_flt_scale);)
 
 			*((int*)ptr) = g_dsp_props.p_flt_scale;
 			break;
 
 		case DSP_REAL_PREC:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_GET, DSP_REAL_PREC) = %d\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_GET, DSP_REAL_PREC) = %d\n",
 			   g_dsp_props.p_real_prec);)
 
 			*((int*)ptr) = g_dsp_props.p_real_prec;
 			break;
 
 		case DSP_REAL_SCALE:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_GET, DSP_REAL_SCALE) = %d\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_GET, DSP_REAL_SCALE) = %d\n",
 			   g_dsp_props.p_real_scale);)
 
 			*((int*)ptr) = g_dsp_props.p_real_scale;
 			break;
 
+		case DSP_DATETIMEFMT:
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_GET, DSP_DATETIMEFMT) = %s\n",
+				dsp_datetimefmt_get());)
+
+			if (len <= 0)
+			{
+				sqsh_set_error( SQSH_E_INVAL, "length must be greater than 0" );
+				return DSP_FAIL;
+			}
+
+			strncpy( (char*)ptr, dsp_datetimefmt_get(), len );
+			break;
+
 		case DSP_DATEFMT:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_GET, DSP_DATEFMT) = %s\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_GET, DSP_DATEFMT) = %s\n",
 				dsp_datefmt_get());)
 
 			if (len <= 0)
@@ -644,74 +693,88 @@ static int dsp_prop_get( prop, ptr, len )
 
 			strncpy( (char*)ptr, dsp_datefmt_get(), len );
 			break;
+
+		case DSP_TIMEFMT:
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_GET, DSP_TIMEFMT) = %s\n",
+				dsp_timefmt_get());)
+
+			if (len <= 0)
+			{
+				sqsh_set_error( SQSH_E_INVAL, "length must be greater than 0" );
+				return DSP_FAIL;
+			}
+
+			strncpy( (char*)ptr, dsp_timefmt_get(), len );
+			break;
 			
 		case DSP_STYLE:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_GET, DSP_STYLE) = %d\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_GET, DSP_STYLE) = %d\n",
 				g_dsp_props.p_style);)
 		
 			*((int*)ptr) = g_dsp_props.p_style;
 			break;
 
-		case DSP_WIDTH: 
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_GET, DSP_WIDTH) = %d\n", 
+		case DSP_WIDTH:
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_GET, DSP_WIDTH) = %d\n",
 				g_dsp_props.p_width);)
 
 			*((int*)ptr) = g_dsp_props.p_width;
 			break;
 
-		case DSP_OUTPUTPARMS: 
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_GET, DSP_OUTPUTPARMS) = %d\n", 
+		case DSP_OUTPUTPARMS:
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_GET, DSP_OUTPUTPARMS) = %d\n",
 				g_dsp_props.p_outputparms);)
 
 			*((int*)ptr) = g_dsp_props.p_outputparms;
 			break;
 		
 		case DSP_COLSEP:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_GET, DSP_COLSEP) = %s\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_GET, DSP_COLSEP) = %s\n",
 				g_dsp_props.p_colsep);)
 
 			strncpy( (char*)ptr, g_dsp_props.p_colsep, len );
 			break;
 
 		case DSP_BCP_COLSEP:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_GET, DSP_BCP_COLSEP) = %s\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_GET, DSP_BCP_COLSEP) = %s\n",
 				g_dsp_props.p_bcp_colsep);)
 
 			strncpy( (char*)ptr, g_dsp_props.p_bcp_colsep, len );
 			break;
 
 		case DSP_BCP_ROWSEP:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_GET, DSP_BCP_ROWSEP) = %s\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_GET, DSP_BCP_ROWSEP) = %s\n",
 				g_dsp_props.p_bcp_rowsep);)
 
 			strncpy( (char*)ptr, g_dsp_props.p_bcp_rowsep, len );
 			break;
 
 		case DSP_BCP_TRIM:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_GET, DSP_BCP_TRIM) = %d\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_GET, DSP_BCP_TRIM) = %d\n",
 				g_dsp_props.p_bcp_trim);)
 
 			*((int*)ptr) = g_dsp_props.p_bcp_trim;
 			break;
 		
 		case DSP_LINESEP:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_GET, DSP_LINESEP) = %s\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_GET, DSP_LINESEP) = %s\n",
 				g_dsp_props.p_linesep);)
 
 			strncpy( (char*)ptr, g_dsp_props.p_linesep, len );
 			break;
 
 		case DSP_XGEOM:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_GET, DSP_XGEOM) = %s\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_GET, DSP_XGEOM) = %s\n",
 				g_dsp_props.p_xgeom);)
 
 			if (len <= 0)
@@ -724,8 +787,8 @@ static int dsp_prop_get( prop, ptr, len )
 			break;
 
 		case DSP_MAXLEN:
-			DBG(sqsh_debug(DEBUG_DISPLAY, 
-				"dsp_prop: dsp_prop(DSP_GET, DSP_MAXLEN) = %d\n", 
+			DBG(sqsh_debug(DEBUG_DISPLAY,
+				"dsp_prop: dsp_prop(DSP_GET, DSP_MAXLEN) = %d\n",
 			   g_dsp_props.p_maxlen);)
 
 			*((int*)ptr) = g_dsp_props.p_maxlen;
