@@ -28,34 +28,59 @@
 #include "sqsh_error.h"
 #include "sqsh_compat.h"
 #include "dsp.h"
+#include "config.h"
+
+#if defined(HAVE_LOCALE_H)
+#include <locale.h>
+#endif
 
 /*-- Current Version --*/
 #if !defined(lint) && !defined(__LINT__)
-static char RCS_Id[] = "$Id: dsp_conv.c,v 1.2 2004/04/10 00:10:30 mpeppler Exp $";
+static char RCS_Id[] = "$Id: dsp_conv.c,v 1.7 2013/07/23 20:57:28 mwesdorp Exp $";
 USE(RCS_Id)
 #endif /* !defined(lint) */
 
+/*
+ * sqsh-2.1.9 - Introduced and remapped global variables sg_datetime_fmt, sg_date_fmt
+ * and sg_time_fmt as well as the functions dsp_datetimefmt_set, dsp_datetimefmt_get,
+ * dsp_datefmt_set, dsp_datefmt_get, dsp_timefmt_set and dsp_timefmt_get to implement
+ * date and time datatype conversions with strftime as already existed for the datetime
+ * and smalldatetime datatypes.
+ * This feature request was filed as bugreport 3603409 on Sourceforge.
+ */
+
 /*-- Local Globals --*/
 static char sg_datetime_fmt[64]  = "\0";
+static char sg_date_fmt[64]      = "\0";
+static char sg_time_fmt[64]      = "\0";
 static int  sg_datetime_len      = -1;
 static int  sg_datetime4_len     = -1;
 static int  sg_date_len          = -1;
 static int  sg_time_len          = -1;
+static int  sg_bigdatetime_len   = -1;
+static int  sg_bigtime_len       = -1;
+#if defined(HAVE_SETLOCALE)
+static char *sg_locale           = NULL;
+#endif
+#if defined(CS_BIGDATETIME_TYPE) && defined(CS_BIGTIME_TYPE)
+static char sg_datetime_def[64]  = "%b %d %Y %l:%M:%S.%q%p";
+static char sg_time_def[64]      = "%l:%M:%S.%q%p";
+#endif
 
 /*-- Prototypes --*/
-static CS_INT dsp_type_len       _ANSI_ARGS(( CS_CONTEXT*, CS_CHAR*, 
+static CS_INT dsp_type_len       _ANSI_ARGS(( CS_CONTEXT*, CS_CHAR*,
                                               CS_DATAFMT*, CS_VOID* ));
 static char*  dsp_datetime_strip _ANSI_ARGS(( CS_INT, char*, int ));
 
 /*
- * dsp_datefmt_set():
+ * dsp_datetimefmt_set():
  *
  * Set the conversion format to be used by the display routine when
  * converting dates.  A format string of NULL or "default" indicates
  * that the default CT-Lib conversion method should be used.  Otherwise
  * a string suitable for use by strftime() should be supplied.
  */
-int dsp_datefmt_set( fmt )
+int dsp_datetimefmt_set( fmt )
     char         *fmt;
 {
     /*
@@ -64,8 +89,9 @@ int dsp_datefmt_set( fmt )
      * we are now changing the format from what it was, these
      * variables should be reset.
      */
-    sg_datetime_len  = -1;
-    sg_datetime4_len = -1;
+    sg_datetime_len    = -1;
+    sg_datetime4_len   = -1;
+    sg_bigdatetime_len = -1;
 
     /*
      * If the format is NULL or "default", then we simply store
@@ -82,18 +108,19 @@ int dsp_datefmt_set( fmt )
      * longer than the size of sg_datetime_fmt, then we quietly
      * trim it to the appropriate length.
      */
-    strncpy( sg_datetime_fmt, fmt, sizeof(sg_datetime_fmt) );
+    strncpy( sg_datetime_fmt, fmt, sizeof(sg_datetime_fmt)-1 );
+    sg_datetime_fmt[sizeof(sg_datetime_fmt)-1] = '\0';
     return DSP_SUCCEED;
 }
 
 /*
- * dsp_datefmt_get():
+ * dsp_datetimefmt_get():
  *
  * Retrieve the conversion format to be used by the display routine when
  * converting dates.  A format string of NULL or "default" indicates
  * that the default CT-Lib conversion method should be used.
  */
-char* dsp_datefmt_get()
+char* dsp_datetimefmt_get()
 {
     if (*sg_datetime_fmt == '\0')
     {
@@ -103,14 +130,139 @@ char* dsp_datefmt_get()
     return sg_datetime_fmt;
 }
 
+
+/*
+ * dsp_datefmt_set():
+ *
+ * sqsh-2.1.9 - Date and time datatype conversion feature.
+ *
+ * Set the conversion format to be used by the display routine when
+ * converting dates.  A format string of NULL or "default" indicates
+ * that the default CT-Lib conversion method should be used.  Otherwise
+ * a string suitable for use by strftime() should be supplied.
+ */
+int dsp_datefmt_set( fmt )
+    char         *fmt;
+{
+    /*
+     * The dsp_datetime_len() function uses these global variables
+     * to store the results of the last time it was called.  Since
+     * we are now changing the format from what it was, these
+     * variables should be reset.
+     */
+    sg_date_len  = -1;
+
+    /*
+     * If the format is NULL or "default", then we simply store
+     * away the format to be used as an empty string.
+     */
+    if (fmt == NULL || strcmp( fmt, "default" ) == 0)
+    {
+        sg_date_fmt[0] = '\0';
+        return DSP_SUCCEED;
+    }
+
+    /*
+     * This is a little cheesy. If the user passes us a string
+     * longer than the size of sg_date_fmt, then we quietly
+     * trim it to the appropriate length.
+     */
+    strncpy( sg_date_fmt, fmt, sizeof(sg_date_fmt)-1 );
+    sg_date_fmt[sizeof(sg_date_fmt)-1] = '\0';
+    return DSP_SUCCEED;
+}
+
+/*
+ * dsp_datefmt_get():
+ *
+ * sqsh-2.1.9 - Date and time datatype conversion feature.
+ *
+ * Retrieve the conversion format to be used by the display routine when
+ * converting dates.  A format string of NULL or "default" indicates
+ * that the default CT-Lib conversion method should be used.
+ */
+char* dsp_datefmt_get()
+{
+    if (*sg_date_fmt == '\0')
+    {
+        return "default";
+    }
+
+    return sg_date_fmt;
+}
+
+/*
+ * dsp_timefmt_set():
+ *
+ * sqsh-2.1.9 - Date and time datatype conversion feature.
+ *
+ * Set the conversion format to be used by the display routine when
+ * converting times.  A format string of NULL or "default" indicates
+ * that the default CT-Lib conversion method should be used.  Otherwise
+ * a string suitable for use by strftime() should be supplied.
+ */
+int dsp_timefmt_set( fmt )
+    char         *fmt;
+{
+    /*
+     * The dsp_time_len() function uses these global variables
+     * to store the results of the last time it was called.  Since
+     * we are now changing the format from what it was, these
+     * variables should be reset.
+     */
+    sg_time_len    = -1;
+    sg_bigtime_len = -1;
+
+    /*
+     * If the format is NULL or "default", then we simply store
+     * away the format to be used as an empty string.
+     */
+    if (fmt == NULL || strcmp( fmt, "default" ) == 0)
+    {
+        sg_time_fmt[0] = '\0';
+        return DSP_SUCCEED;
+    }
+
+    /*
+     * This is a little cheesy. If the user passes us a string
+     * longer than the size of sg_time_fmt, then we quietly
+     * trim it to the appropriate length.
+     */
+    strncpy( sg_time_fmt, fmt, sizeof(sg_time_fmt)-1 );
+    sg_time_fmt[sizeof(sg_time_fmt)-1] = '\0';
+    return DSP_SUCCEED;
+}
+
+/*
+ * dsp_timefmt_get():
+ *
+ * sqsh-2.1.9 - Date and time datatype conversion feature.
+ *
+ * Retrieve the conversion format to be used by the display routine when
+ * converting dates.  A format string of NULL or "default" indicates
+ * that the default CT-Lib conversion method should be used.
+ */
+char* dsp_timefmt_get()
+{
+    if (*sg_time_fmt == '\0')
+    {
+        return "default";
+    }
+
+    return sg_time_fmt;
+}
+
 /*
  * dsp_datetime_len():
  *
  * Calculates the maximum number of characters required to store
  * a date in a string format of the current locale.  This function
- * is similar to dsp_money_len() except that every month is 
+ * is similar to dsp_money_len() except that every month is
  * converted to verify if particular month names are longer than
  * others.
+ *
+ * sqsh-2.1.9 - Also implement date and time datatype conversions.
+ * sqsh-2.3   - Improve handling of bigdatetime and bigtime data types.
  */
 CS_INT dsp_datetime_len( ctxt, type )
     CS_CONTEXT   *ctxt;
@@ -125,6 +277,30 @@ CS_INT dsp_datetime_len( ctxt, type )
     int             max_len;
     int             len;
     char           *fmt;
+#if defined(CS_BIGDATETIME_TYPE) && defined(CS_BIGTIME_TYPE)
+    char           *conv_fmt;
+#endif
+#if  defined(HAVE_LOCALE_H) && defined(HAVE_SETLOCALE)
+    char           *locale;
+
+    /*
+     * sqsh-2.3 : If the locale was changed recently, we have to reset
+     * the cached type len values.
+     */
+    locale = setlocale (LC_ALL, NULL);
+    if ( sg_locale != locale )
+    {
+        DBG(sqsh_debug(DEBUG_DISPLAY, "locale changed to %s\n", locale);)
+
+        sg_locale = locale;
+        sg_datetime_len      = -1;
+        sg_datetime4_len     = -1;
+        sg_date_len          = -1;
+        sg_time_len          = -1;
+        sg_bigdatetime_len   = -1;
+        sg_bigtime_len       = -1;
+    }
+#endif
 
     /*
      * Check to see if we have cached the value from the last time
@@ -155,6 +331,20 @@ CS_INT dsp_datetime_len( ctxt, type )
     }
 #endif
 
+#if defined(CS_BIGDATETIME_TYPE)
+    if (type == CS_BIGDATETIME_TYPE && sg_bigdatetime_len != -1)
+    {
+        return sg_bigdatetime_len;
+    }
+#endif
+
+#if defined(CS_BIGTIME_TYPE)
+    if (type == CS_BIGTIME_TYPE && sg_bigtime_len != -1)
+    {
+        return sg_bigtime_len;
+    }
+#endif
+
     /*
      * If the user is using the default CT-Lib conversion format then
      * we try a brute force test of all of the months in the year,
@@ -162,15 +352,12 @@ CS_INT dsp_datetime_len( ctxt, type )
      */
     max_len = 0;
 
-	/* The user defined conversion with strftime, etc. is not implemented
-	   for the CS_DATE_TYPE and CS_TIME_TYPE datatypes */
-    if (*sg_datetime_fmt == '\0' || 
+    if ((type == CS_DATETIME_TYPE  && *sg_datetime_fmt == '\0')
+     || (type == CS_DATETIME4_TYPE && *sg_datetime_fmt == '\0')
 #if defined(CS_DATE_TYPE)
-		(type == CS_DATE_TYPE || type == CS_TIME_TYPE)
-#else
-		0
+     || (type == CS_DATE_TYPE && *sg_date_fmt == '\0')
 #endif
-		)
+       )
     {
         dt_fmt.datatype  = type;
         dt_fmt.maxlength = sizeof(CS_DATETIME);
@@ -189,21 +376,52 @@ CS_INT dsp_datetime_len( ctxt, type )
         for (i = 1; i <= 12; i++)
         {
             /*-- Build a made-up day of the month --*/
-            sprintf( dt_buf, "%d/12/1997 11:59:53:123PM", i );
 
+            switch (type)
+            {
+                case CS_DATETIME_TYPE:
+                    sprintf( dt_buf, "%d/12/1997 11:59:53.123PM", i );
+                    break;
+                case CS_DATETIME4_TYPE:
+                    sprintf( dt_buf, "%d/12/1997 11:59PM", i );
+                    break;
+#if defined(CS_DATE_TYPE)
+                case CS_DATE_TYPE:
+                    sprintf( dt_buf, "%d/12/1997", i );
+                    break;
+#endif
+                default:
+                    sprintf( dt_buf, "%d/12/1997 11:59:53PM", i );
+                    break;
+            }
             len = dsp_type_len( ctxt, (CS_CHAR*)dt_buf, &dt_fmt, &dt );
 
             /*-- Keep track of the longest date encountered --*/
             max_len = max(len, max_len);
         }
     }
+#if defined(CS_TIME_TYPE)
+    else if (type == CS_TIME_TYPE && *sg_time_fmt == '\0')
+    {
+        dt_fmt.datatype  = type;
+        dt_fmt.maxlength = sizeof(CS_DATETIME);
+        dt_fmt.locale    = NULL;
+        dt_fmt.format    = CS_FMT_UNUSED;
+        dt_fmt.scale     = 0;
+        dt_fmt.precision = 0;
+        sprintf( dt_buf, "11:59:53.123PM" );
+        max_len = dsp_type_len( ctxt, (CS_CHAR*)dt_buf, &dt_fmt, &dt );
+    }
+#endif
     else
     {
         /*
          * If the user has supplied a display format to be used in
          * place of the CT-Lib format, then we have a lot of work
-         * to do; beginning wth building a reasonable UNIX time
+         * to do; beginning with building a reasonable UNIX time
          * structure.
+         * sqsh-2.3 : Note that CS_BIGDATETIME_TYPE and CS_BIGTIME_TYPE
+         * will use a default format here, if one is not supplied.
          */
         tm.tm_sec   = 59;
         tm.tm_min   = 59;
@@ -216,10 +434,37 @@ CS_INT dsp_datetime_len( ctxt, type )
 
         /*
          * Strip down the format string as defined by the CS_DATETIME
-         * datatype, and replace the ms (%u) with the longest possible
+         * datatype, and replace the ms (%q) with the longest possible
          * number.
          */
-        fmt = dsp_datetime_strip( type, sg_datetime_fmt, 999 );
+        switch (type)
+        {
+#if defined(CS_DATE_TYPE)
+            case CS_DATE_TYPE:
+                fmt = dsp_datetime_strip( type, sg_date_fmt, 999 );
+                break;
+#endif
+#if defined(CS_TIME_TYPE)
+            case CS_TIME_TYPE:
+                fmt = dsp_datetime_strip( type, sg_time_fmt, 999 );
+                break;
+#endif
+#if defined(CS_BIGDATETIME_TYPE)
+            case CS_BIGDATETIME_TYPE:
+                conv_fmt = sg_datetime_fmt[0] == '\0' ? sg_datetime_def : sg_datetime_fmt;
+                fmt = dsp_datetime_strip( type, conv_fmt, 999999 );
+                break;
+#endif
+#if defined(CS_BIGTIME_TYPE)
+            case CS_BIGTIME_TYPE:
+                conv_fmt = sg_time_fmt[0] == '\0' ? sg_time_def : sg_time_fmt;
+                fmt = dsp_datetime_strip( type, conv_fmt, 999999 );
+                break;
+#endif
+            default:
+                fmt = dsp_datetime_strip( type, sg_datetime_fmt, 999 );
+                break;
+        }
 
         max_len   = 0;
         max_month = 0;
@@ -270,31 +515,42 @@ CS_INT dsp_datetime_len( ctxt, type )
         }
     }
 
-    DBG(sqsh_debug(DEBUG_DISPLAY, 
-                   "dsp_datetime_len: %s = %d chars\n", 
-                   (type == CS_DATETIME_TYPE)?"CS_DATETIME":"CS_SMALLDATETIME",
-                   max_len);)
+    DBG(sqsh_debug(DEBUG_DISPLAY,
+                   "dsp_datetime_len: %s (type %d) = %d chars\n",
+                   (type == CS_DATETIME_TYPE)?"CS_DATETIME":"Other DATE/TIME type",
+                   type, max_len);)
 
-	switch(type) {
-	  case CS_DATETIME_TYPE:
-        sg_datetime_len = max_len;
-		break;
-	  case CS_DATETIME4_TYPE:
-		sg_datetime4_len = max_len;
-		break;
+    switch (type) {
+        case CS_DATETIME_TYPE:
+            sg_datetime_len = max_len;
+            break;
+
+        case CS_DATETIME4_TYPE:
+            sg_datetime4_len = max_len;
+            break;
 #if defined(CS_DATE_TYPE)
-	  case CS_DATE_TYPE:
-		sg_date_len = max_len;
-		break;
+        case CS_DATE_TYPE:
+            sg_date_len = max_len;
+            break;
 #endif
 #if defined(CS_TIME_TYPE)
-	  case CS_TIME_TYPE:
-		sg_time_len = max_len;
-		break;
+        case CS_TIME_TYPE:
+            sg_time_len = max_len;
+            break;
+#endif
+#if defined(CS_BIGDATETIME_TYPE)
+        case CS_BIGDATETIME_TYPE:
+            sg_bigdatetime_len = max_len;
+            break;
+#endif
+#if defined(CS_BIGTIME_TYPE)
+        case CS_BIGTIME_TYPE:
+            sg_bigtime_len = max_len;
+            break;
 #endif
     }
 
-/*	fprintf(stderr, "type = %d, len = %d\n", type, max_len); */
+/*  fprintf(stderr, "type = %d, len = %d\n", type, max_len); */
 
     return (CS_INT)max_len;
 }
@@ -306,30 +562,56 @@ CS_INT dsp_datetime_len( ctxt, type )
  * of length, len according to the conversion style as defined in
  * dsp_datetime_fmt().  This function returns CS_SUCCEED up success or
  * CS_FAIL upon failure (duh!).
+ *
+ * sqsh-2.1.9 - Also implement the date and time datatype conversions.
  */
-CS_RETCODE dsp_datetime_conv( ctx, dt_fmt, dt, buf, len )
+CS_RETCODE dsp_datetime_conv( ctx, dt_fmt, dt, buf, len, type )
     CS_CONTEXT  *ctx;     /* Context */
     CS_DATAFMT  *dt_fmt;  /* Date format */
     CS_VOID     *dt;      /* Pointer to date */
     CS_CHAR     *buf;     /* Buffer */
     CS_INT       len;     /* Length */
+    CS_INT       type;    /* CS_TYPE */
 {
     struct tm   tm;
     char       *fmt;
     CS_DATEREC  dr;
     CS_DATAFMT  cs_fmt;
+    char       *conv_fmt;
+
+
+    switch (type)
+    {
+#if defined(CS_DATE_TYPE)
+        case CS_DATE_TYPE:
+            conv_fmt = sg_date_fmt;
+            break;
+#endif
+#if defined(CS_TIME_TYPE)
+        case CS_TIME_TYPE:
+            conv_fmt = sg_time_fmt;
+            break;
+#endif
+#if defined(CS_BIGDATETIME_TYPE)
+        case CS_BIGDATETIME_TYPE:
+            conv_fmt = sg_datetime_fmt[0] == '\0' ? sg_datetime_def : sg_datetime_fmt;
+            break;
+#endif
+#if defined(CS_BIGTIME_TYPE)
+        case CS_BIGTIME_TYPE:
+            conv_fmt = sg_time_fmt[0] == '\0' ? sg_time_def : sg_time_fmt;
+            break;
+#endif
+        default:
+            conv_fmt = sg_datetime_fmt;
+            break;
+    }
 
     /*
      * If the user has not specified a format, then we let ct-lib
      * do its thing.  This seems to be OK for most platforms.
      */
-    if (*sg_datetime_fmt == '\0' ||
-#if defined(CS_DATE_TYPE)
-		(dt_fmt->datatype == CS_DATE_TYPE || dt_fmt->datatype == CS_TIME_TYPE)
-#else
-		0
-#endif
-		)
+    if (*conv_fmt == '\0')
     {
         cs_fmt.datatype  = CS_CHAR_TYPE;
         cs_fmt.locale    = NULL;
@@ -345,8 +627,7 @@ CS_RETCODE dsp_datetime_conv( ctx, dt_fmt, dt, buf, len )
                         (CS_VOID*)buf,         /* String */
                         (CS_INT*)NULL ) != CS_SUCCEED)
         {
-            fprintf( stderr, 
-                "dsp_datetime_conv: cs_convert(DATETIME->CHAR) failed\n" );
+            fprintf( stderr, "dsp_datetime_conv: cs_convert(DATETIME->CHAR) failed\n" );
             return CS_FAIL;
         }
 
@@ -379,8 +660,12 @@ CS_RETCODE dsp_datetime_conv( ctx, dt_fmt, dt, buf, len )
      * type of date that we are processing and replace the ms
      * field if it exists.
      */
-    fmt = dsp_datetime_strip( dt_fmt->datatype, sg_datetime_fmt, 
-                              (int)dr.datemsecond );
+#if defined(CS_BIGDATETIME_TYPE) && defined(CS_BIGTIME_TYPE)
+    if (dt_fmt->datatype == CS_BIGDATETIME_TYPE || dt_fmt->datatype == CS_BIGTIME_TYPE)
+        fmt = dsp_datetime_strip( dt_fmt->datatype, conv_fmt, (int) dr.datesecfrac );
+    else
+#endif
+        fmt = dsp_datetime_strip( dt_fmt->datatype, conv_fmt, (int) dr.datemsecond );
 
     /*
      * According to the strftime(3C) documentation, there is no real
@@ -426,12 +711,12 @@ CS_INT dsp_money_len( ctxt )
      */
     len = dsp_type_len( ctxt, (CS_CHAR*)"-922337203685477.5808",
                               &mon_fmt, (CS_VOID*)&mon );
-    
+
     if (len < 19)
     {
         len = 19;
     }
-    
+
     DBG(sqsh_debug(DEBUG_DISPLAY, "dsp_money_len: CS_MONEY = %d chars\n",
                         len);)
 
@@ -550,7 +835,7 @@ static char* dsp_datetime_strip( type, fmt, ms )
 
     /*
      * Now, we want to keep date format that the caller specified,
-     * but we want to strip out the milisecond (%u) and the 
+     * but we want to strip out the millisecond (%q) and the
      * delimeter for small and regular datetimes ([]).
      */
     for (cp = new_fmt; *fmt != '\0'; ++fmt)
@@ -558,11 +843,21 @@ static char* dsp_datetime_strip( type, fmt, ms )
         switch (*fmt)
         {
             case '%':
-                if (*(fmt + 1) == 'u')
+                if (*(fmt + 1) == 'q')
                 {
-                    fmt += 2;
-                    sprintf( cp, "%03d", ms );
-                    cp += 3;
+                    fmt += 1;
+#if defined(CS_BIGDATETIME_TYPE) && defined(CS_BIGTIME_TYPE)
+                    if (type == CS_BIGDATETIME_TYPE || type == CS_BIGTIME_TYPE)
+                    {
+                        sprintf( cp, "%06d", ms );
+                        cp += 6;
+                    }
+                    else
+#endif
+                    {
+                        sprintf( cp, "%03d", ms );
+                        cp += 3;
+                    }
                 }
                 else
                 {
@@ -583,7 +878,7 @@ static char* dsp_datetime_strip( type, fmt, ms )
 
             case ']':
                 break;
-            
+
             default:
                 *cp++ = *fmt;
         }
