@@ -29,18 +29,14 @@
 #include "sqsh_error.h"
 #include "sqsh_stdin.h"
 #include "sqsh_buf.h"
+#include "sqsh_readline.h"
 
 /*-- Current Version --*/
 #if !defined(lint) && !defined(__LINT__)
-static char RCS_Id[] = "$Id: sqsh_buf.c,v 1.3 2010/01/26 15:03:50 mwesdorp Exp $" ;
+static char RCS_Id[] = "$Id: sqsh_buf.c,v 1.4 2012/03/14 09:17:51 mwesdorp Exp $" ;
 USE(RCS_Id)
 #endif /* !defined(lint) */
 
-#if defined(USE_READLINE)
-#include <readline/readline.h>
-extern void add_history();
-#endif /* USE_READLINE */
-
 /*
  * Note: This whole module is quite a bit different from all other
  * sqsh_*.c modules, in that it provides no real data for itself.
