@@ -34,9 +34,10 @@
  * The maximum length of a password.  I just pulled this one out of
  * the air.
  * sqsh-2.1.6 - Increased value from 12 to 30
+ * sqsh-2.4   - Increased value from 30 to 64
  */
 #ifndef SQSH_PASSLEN
-#define SQSH_PASSLEN    30
+#define SQSH_PASSLEN    64
 #endif
 
 /*
@@ -128,6 +129,19 @@
 #define SQSH_MAXFD         256
 #endif
 
+#if defined(USE_READLINE)
+/*
+ * sqsh-2.1.8 - Define the default query that will be executed in
+ * case keyword_dynamic is enabled and an interactive connection is setup
+ * to a Sybase ASE or Microsoft MSSQL database server, or the database
+ * context is changed due to a "use" command.
+ *
+ */
+#  ifndef SQSH_KEYQUERY
+#  define SQSH_KEYQUERY    "select name from sysobjects order by name"
+#  endif
+#endif
+
 /*
  * End user configuration section
  */
@@ -135,7 +149,7 @@
 /*
  * Current version number.
  */
-#define SQSH_VERSION     "sqsh-2.1.7"
+#define SQSH_VERSION     "sqsh-2.5.16.1"
 
 #if !defined(__ansi__)
 #  if defined(__STDC__) || defined(STDC_HEADERS) || defined(PROTOTYPES)
