@@ -28,7 +28,7 @@
 
 /*-- Current Version --*/
 #if !defined(lint) && !defined(__LINT__)
-static char RCS_Id[] = "$Id: sqsh_global.c,v 1.6 2010/01/26 15:03:50 mwesdorp Exp $" ;
+static char RCS_Id[] = "$Id: sqsh_global.c,v 1.11 2014/01/19 10:26:00 mwesdorp Exp $" ;
 USE(RCS_Id)
 #endif /* !defined(lint) */
 
@@ -50,10 +50,23 @@ alias_t    *g_alias        = NULL;
 char       *g_password     = NULL;
 int         g_password_set = False;
 char       *g_lock         = NULL;
-char       *g_copyright    = "Copyright (C) 1995-2001 Scott C. Gray\nPortions Copyright (C) 2004-2010 Michael Peppler";
+char       *g_copyright    = "Copyright (C) 1995-2001 Scott C. Gray\nPortions Copyright (C) 2004-2014 Michael Peppler and Martin Wesdorp";
 char       *g_version      = SQSH_VERSION;
 dsp_desc_t *g_do_cols[64];
 int         g_do_ncols     = 0;
 funcarg_t   g_func_args[64];
-int         g_func_nargs   = 0;  
-int         g_interactive  = False;  
+int         g_func_nargs   = 0;
+int         g_interactive  = False;
+
+#if defined(HAVE_LOCALE_H)
+    struct lconv  *g_lconv = NULL;
+#else
+    void *g_lconv = NULL;
+#endif
+
+/*
+ * sqsh-2.5 - Initialize variables for p2f feature.
+ */
+FILE       *g_p2f_fp       = NULL;
+int         g_p2fc         = 0;
+
