@@ -35,6 +35,12 @@
 #include "dsp.h"
 #include "sqsh_func.h"
 
+#include "config.h"
+
+#if defined(HAVE_LOCALE_H)
+    #include <locale.h>
+#endif
+
 /* g_cs_ver: This is the value of CS_VERSION_xxx. Stored in a global
  *          because it is needed to figure out the correct BLK_VERSION_xxx
  *          value to use in cmd_blk.c
@@ -141,7 +147,7 @@ extern char       *g_version ;
 
 /*
  * g_password & g_lock: Contains the current value of the regular database
- *              password and the session lock password. 
+ *              password and the session lock password.
  */
 extern int         g_password_set;
 extern char       *g_password;
@@ -152,4 +158,20 @@ extern char       *g_lock;
  */
 extern int         g_interactive;
 
+/*
+ * g_lconv: sqsh-2.3 - Do locale conversion of numeric/decimal/money datatypes.
+ */
+#if defined(HAVE_LOCALE_H)
+    extern struct lconv *g_lconv;
+#else
+    extern void *g_lconv;
+#endif
+
+/*
+ * sqsh-2.5 - New feature: Print to file from message handler after $p2faxm
+ * number of printed messages to screen.
+ */
+extern FILE       *g_p2f_fp; /* Print to file filepointer */
+extern int         g_p2fc;   /* Print to file count */
+
 #endif
