@@ -42,7 +42,7 @@
 
 /*-- Current Version --*/
 #if !defined(lint) && !defined(__LINT__)
-static char RCS_Id[] = "$Id: sqsh_main.c,v 1.17 2010/02/17 11:35:06 mwesdorp Exp $";
+static char RCS_Id[] = "$Id: sqsh_main.c,v 1.23 2013/04/25 14:09:47 mwesdorp Exp $";
 USE(RCS_Id)
 #endif /* !defined(lint) */
 
@@ -145,7 +145,6 @@ main( argc, argv )
     char          *batch_failcount;
     char          *exit_failcount;
     char          *exit_value;
-    char          *keyword_file;
     char          *term_title;
     char           str[512];
     int            ch;
@@ -162,6 +161,15 @@ main( argc, argv )
     int            i;
     varbuf_t      *exp_buf;
 
+    /*
+     * sqsh-2.2.0 - Variables used by password handling option
+     * moved here.
+     */
+    char           buf[MAXPWD];
+    char          *p;
+    int            fdin, fdout;
+
+
     /*
      * If termios.h defines TIOCGWINSZ, then we need to declare a
      * structure in which to retrieve the current window size.
@@ -216,6 +224,8 @@ main( argc, argv )
         else
         {
             rcfile = NULL;
+            /* sqsh-2.1.9 - Set rcfile environment variable also to NULL */
+            env_set( g_env, "rcfile", rcfile );
         }
     }
     else
@@ -267,7 +277,7 @@ main( argc, argv )
             if (access( cptr, R_OK ) != -1)
             {
                 env_set( g_env, "cur_rcfile", cptr );
-                if((jobset_run( g_jobset, "\\loop -n $cur_rcfile", &exit_status)) 
+                if((jobset_run( g_jobset, "\\loop -n $cur_rcfile", &exit_status))
                     == -1 )
                 {
                     fprintf( stderr, "\\loop: %s\n", sqsh_get_errstr() );
@@ -283,6 +293,8 @@ main( argc, argv )
 
             cptr = strtok( NULL, ":\n\t\r" );
         }
+        /* sqsh-2.1.9 - Remove temporary environment variable cur_rcfile */
+        env_remove( g_env, "cur_rcfile", 0);
         varbuf_destroy(exp_buf);
     }
 
@@ -297,7 +309,7 @@ main( argc, argv )
         "a:A:bBc;C:d:D:eE:f:G:hH:i:I:J:k:K:l:L:m:n:N:o:pP;Q:r;R:s:S:t;T:U:vV;w:Xy:z:Z;\250:" )) != EOF)
     {
         ret = 0;
-        switch (ch) 
+        switch (ch)
         {
             case 'a' :
                 ret = env_set( g_env, "thresh_exit", sqsh_optarg );
@@ -365,7 +377,6 @@ main( argc, argv )
                 read_file   = True;
                 show_banner = False;
                 ret = env_set( g_env, "script", sqsh_optarg );
-                ret = env_set( g_internal_env, "0", sqsh_optarg );
                 break;
             case 'I' :
                 ret = env_set( g_env, "interfaces", sqsh_optarg );
@@ -436,19 +447,19 @@ main( argc, argv )
                     while( *sqsh_optarg != '\0' )
                         *sqsh_optarg++ = ' ';
                 }
-                
+
                 break;
             case 'Q' : /* sqsh-2.1.6 */
                 ret = env_set( g_env, "query_timeout", sqsh_optarg );
                 break;
             case 'r' :
-		/*
-		 * The alternative sqshrc file should already have been
-		 * processed above. Note that the -r option should be the
-		 * first option on the command line, otherwise the option
-		 * is ignored, also the option and the filename should be
-		 * separated by at leat a blank space.
-		*/
+                /*
+                 * The alternative sqshrc file should already have been
+                 * processed above. Note that the -r option should be the
+                 * first option on the command line, otherwise the option
+                 * is ignored, also the option and the filename should be
+                 * separated by at leat a blank space.
+                */
                 ret = True;
                 break;
             case 'R' : /* sqsh-2.1.6 */
@@ -508,19 +519,15 @@ main( argc, argv )
             case '\250' :
 #if defined(SQSH_HIDEPWD)
                 {
-		  /*
- 		   * sqsh-2.1.7 - Incorporated patch by David Wood
-		   * to solve a problem with pipes already in use. (Patch-id 2607434)
-		   * The actual pipe file descriptors will now be passed on with the \250 option.
-		  */
-                  char  buf[MAXPWD];
-                  char *p;
-                  int   fdin, fdout;
-
+                  /*
+                   * sqsh-2.1.7 - Incorporated patch by David Wood
+                   * to solve a problem with pipes already in use. (Patch-id 2607434)
+                   * The actual pipe file descriptors will now be passed on with the \250 option.
+                  */
                   memset(buf, 0, MAXPWD);
                   if (sqsh_optarg != NULL)
                     strcpy (buf, sqsh_optarg);
-                  if (p = strchr(buf, '/')) {
+                  if ((p = strchr(buf, '/')) != NULL) {
                     *p    = '\0';
                     fdin  = atoi(buf);
                     fdout = atoi(p+1);
@@ -556,7 +563,7 @@ main( argc, argv )
          * Check the results from whichever variable we attempted
          * to set.
          */
-        if (ret == False) 
+        if (ret == False)
         {
             fprintf( stderr, "sqsh: -%c: %s\n", ch, sqsh_get_errstr() );
             sqsh_exit( 255 );
@@ -582,7 +589,7 @@ main( argc, argv )
     {
         if (sqsh_optind > 1)
         {
-            argv[sqsh_optind-1] = argv[0];
+            env_get (g_env, "script", &(argv[sqsh_optind-1])) ;
         }
         g_func_args[g_func_nargs].argc = argc - sqsh_optind + 1;
         g_func_args[g_func_nargs].argv = &(argv[sqsh_optind-1]);
@@ -610,7 +617,7 @@ main( argc, argv )
         sqsh_exit( 255 );
     }
     if (stdin_tty && stdout_tty)
-	    g_interactive = True;
+        g_interactive = True;
 
 #if defined(TIOCGWINSZ)
     /*
@@ -620,24 +627,25 @@ main( argc, argv )
      */
     if (set_width == False && stdout_tty)
     {
-	/* Check to see if the width has been set via the sqshrc file.
-	   To do this we get the current setting - if it is != 80 then
-	   the sqshrc file had a \set width directive, which we don't want
-	   to override here. */
-	char *w;
-	env_get( g_env, "width", &w);
-	if(!w || atoi(w) == 80) {
-	    if (ioctl(fileno(stdout), TIOCGWINSZ, &ws ) != -1) {
-		sprintf( str, "%d", ws.ws_col );
-		env_set( g_env, "width", str );
-		
-		DBG(sqsh_debug(DEBUG_SCREEN,"sqsh_main: Screen width = %d\n",
-			       ws.ws_col);)
-	    } else {
+        /* Check to see if the width has been set via the sqshrc file.
+         * To do this we get the current setting - if it is != 80 then
+         * the sqshrc file had a \set width directive, which we don't want
+         * to override here.
+         */
+        char *w;
+        env_get( g_env, "width", &w);
+        if(!w || atoi(w) == 80) {
+            if (ioctl(fileno(stdout), TIOCGWINSZ, &ws ) != -1) {
+                sprintf( str, "%d", ws.ws_col );
+                env_set( g_env, "width", str );
+
+                DBG(sqsh_debug(DEBUG_SCREEN,"sqsh_main: Screen width = %d\n",
+                    ws.ws_col);)
+            } else {
             DBG(sqsh_debug(DEBUG_SCREEN,"sqsh_main: ioctl(%d,TIOCGWINSZ): %s\n",
                            (int)fileno(stdout), strerror(errno));)
-	    }
-	}
+            }
+        }
 
 #if defined(SIGWINCH)
         if (stdout_tty)
@@ -691,7 +699,7 @@ main( argc, argv )
      * banner.
      */
     env_get( g_env, "banner", &banner );
-    if (show_banner && (banner == NULL || *banner == '1') && 
+    if (show_banner && (banner == NULL || *banner == '1') &&
         g_interactive)
     {
         printf( "%s ", g_version );
@@ -738,7 +746,7 @@ main( argc, argv )
         env_get( g_env, "history", &history );
 
         /*
-         * If a history file has been defined, then we want to 
+         * If a history file has been defined, then we want to
          * expand its contents. This will allow folks to have
          * a different history file for each server.
          */
@@ -768,32 +776,10 @@ main( argc, argv )
             env_set( g_env, "histnum", str );
         }
 
-        /*
-         * If the user has requested some form of keyword completion
-         * then attempt to read the contents of the keywords file.
-         * sqsh-2.1.6 feature - Expand keyword_file variable
-         */
-        env_get( g_env, "keyword_file", &keyword_file );
-        if ( keyword_file != NULL && *keyword_file != '\0')
-        {
-            exp_buf = varbuf_create( 512 );
-            if (exp_buf == NULL)
-            {
-                fprintf( stderr, "sqsh: %s\n", sqsh_get_errstr() );
-                sqsh_exit( 255 );
-            }
-            if (sqsh_expand( keyword_file, exp_buf, 0 ) != False)
-                sqsh_readline_read( varbuf_getstr( exp_buf) );
-            else
-                fprintf( stderr, "sqsh: Error expanding $keyword_file: %s\n",
-                       sqsh_get_errstr() );
-            varbuf_destroy( exp_buf );
-        }
-
         /*
          * Initialize the readline "sub-system".  This basically consists
          * of installing handlers for readline keyword completion and
-         * sucking in the readline history file.
+         * sucking in the completion keyword list and the readline history file.
          */
         sqsh_readline_init();
     }
@@ -882,6 +868,7 @@ main( argc, argv )
      */
     sqsh_exit( 0 );
     /* NOTREACHED */
+    return (0);
 }
 
 #if defined(TIOCGWINSZ) && defined(SIGWINCH)
@@ -891,7 +878,7 @@ main( argc, argv )
  *
  * This function is called whenever a SIGWINCH (window size change)
  * signal is recieved during the life of sqsh.  Unfortunately, this
- * function calls quite a few functions that are known not to be 
+ * function calls quite a few functions that are known not to be
  * signal safe, but I am willing to accept the risk.  Thanks
  * to David Whitemarsh <djw@accessio.demon.co.uk> for supplying
  * this code.
@@ -914,7 +901,7 @@ static void sigwinch_handler( sig, user_data )
     if (isatty( fileno(stdout) ))
         ctty_fd = fileno(stdout);
     else {
-        
+
         /*
          * Attempt to grab the path to our controlling tty.  If we can't
          * find it, then we have to give up.
@@ -987,7 +974,7 @@ static void print_usage()
             sprintf( str, " [%s]", sg_flags[i].flag );
         else
             sprintf( str, " [%s %s]", sg_flags[i].flag, sg_flags[i].arg );
-        
+
         len = strlen( str );
 
         if( (line_len + len) > 68 ) {
@@ -1025,60 +1012,117 @@ static void print_usage()
  * sqsh-2.1.7 - Incorporated patch by David Wood
  * to solve a problem with pipes already in use. (Patch-id 2607434)
  * The actual pipe file descriptors will now be passed on with the \250 option.
+ * sqsh-2.2.0 - Function reworked.
 */
 static void hide_password (argc, argv)
   int   argc;
   char *argv[];
 {
-  int  i, j;
-  int  filedes[2];
-  char buf[32];
-  char nullpwd[2];
-  char *pwd = NULL;
+  int    i, j;
+  char   buf[32];
+  int    filedes[2];
+  char   nullpwd[2];
+  pid_t  pid;
+  char  *pwd = NULL;
+  int    status;
 
-  for (i = 1; i < argc; ++i) {
-    if (*(argv[i]) == '-' && *(argv[i] + 1) == 'P') {
-      if (*(argv[i]+2)) {
-        pwd = (argv[i]+2); /* Password passed on as: "sqsh -SSYBASE -Usa -Pxxxxxx" */
-      } else if (i + 1 < argc && *(argv[i + 1]) != '-') {
-        pwd = argv[i + 1]; /* Password passed on as: "sqsh -SSYBASE -Usa -P xxxxxx" */
-        /* Reshuffle the argv list to remove the password string */
-        for (j = i + 1; j < argc - 1; j++)
-          argv[j] = argv[j + 1];
-        argv[j] = NULL;
-        argc--;
+
+  nullpwd[0] = '\n';
+  nullpwd[1] = '\0';
+
+  /*
+   * Loop through the list of arguments only once and skip all intermediate -P
+   * entries, but remember the last password specified.
+  */
+  for (i = 0, j = 0; argv[i] != NULL; ++i)
+  {
+    if (*(argv[i]) == '-' && *(argv[i]+1) == 'P')
+    {
+      /*
+       * New password parameter encounterd.
+      */
+      pwd = NULL;
+      if (*(argv[i]+2) != '\0')
+      {
+        /*
+         * Password passed on as: "sqsh -SSYBASE -Usa -Pxxxxxx" , or as -P-
+        */
+        pwd = (argv[i]+2);
+      }
+      else if ((i+1 < argc) && (*(argv[i+1]) != '-' || (*(argv[i+1]) == '-' && *(argv[i+1]+1) == '\0')))
+      {
+        /*
+         * Password passed on as: "sqsh -SSYBASE -Usa -P xxxxxx" , or as -P -
+        */
+        pwd = argv[++i];
       }
       if (pwd == NULL || (pwd != NULL && strlen(pwd) == 0))
       {
-        nullpwd[0] = '\n';
-        nullpwd[1] = '\0';
-        pwd = nullpwd; /* Empty (NULL) password passed on as:
-			* "sqsh -SSYBASE -Usa -P " or "sqsh -SSYBASE -Usa -P '' " */
-      }
-
-      /* Create the pipe... */
-      if (pipe(filedes) == -1) {
-        perror("sqsh: Error: Can't pipe()");
-        return;
-      }
-      sprintf(buf, "-%c%d/%d", '\250', filedes[0], filedes[1]);
-      argv[i] = buf;
-
-      if (fork()) {
-        /* Re-execute ourselves, with the modified argv[] */
-        execvp(argv[0], argv);
-        /* Not reached */
-      } else {
-        /* ... and write the password to the pipe */
-        if (write(filedes[1], pwd, strlen(pwd)) != strlen(pwd)) {
-          fprintf(stderr, "sqsh: Error: Failed to write password to pipe (filedes=%d)\n", filedes[1]);
-          sqsh_exit(255);
-        }
-        close(filedes[0]);
-        close(filedes[1]);
-        sqsh_exit(0);
+        /*
+         * Empty (NULL) password passed on as:
+         * "sqsh -SSYBASE -Usa -P " or "sqsh -SSYBASE -Usa -P '' "
+        */
+        pwd = nullpwd;
       }
     }
+    else
+      argv[j++] = argv[i];
+  }
+
+  /*
+   * If pwd == NULL then no -P was specified as argument and we do not
+   * have to hide anything from the argument list.
+  */
+  if (pwd == NULL)
+  {
+    return;
+  }
+
+  /*
+   * Create the pipe.
+  */
+  if (pipe (filedes) == -1)
+  {
+    perror ("sqsh: Error: Can't pipe()");
+    return;
+  }
+  sprintf (buf, "-%c%d/%d", '\250', filedes[0], filedes[1]);
+  argv[j++] = buf;
+  argv[j]   = NULL;
+
+  if ((pid = fork()) != 0)
+  {
+    /*
+     * sqsh-2.2.0 - This code is executed by the parent process.
+     * Wait for the child process to finish execution before we continue.
+     */
+    (void) waitpid (pid, &status, 0);
+    /*
+     * Re-execute ourselves in the parent process, with the modified argv[] list.
+     */
+    if (WIFEXITED(status) != 0 && WEXITSTATUS(status) == 0)
+      (void) execvp (argv[0], argv);
+      /* Not reached */
+    else
+    {
+      fprintf (stderr, "sqsh: Error: Processing of function hide_password failed unexpectedly\n");
+      sqsh_exit (255);
+    }
+  }
+  else
+  {
+    /*
+     * The child process writes the password to the pipe, closes the pipe
+     * and exits.
+    */
+    if ((int) write (filedes[1], pwd, strlen(pwd)) != (int) strlen(pwd))
+    {
+      fprintf (stderr, "sqsh: Error: Failed to write password to pipe (filedes=%d)\n", filedes[1]);
+      sqsh_exit (255);
+    }
+    (void) close (filedes[0]);
+    (void) close (filedes[1]);
+    sqsh_exit (0);
   }
 }
 
