@@ -25,12 +25,40 @@
 #ifndef sqsh_readline_h_included
 #define sqsh_readline_h_included
 
+#if defined(USE_READLINE)
+#include <readline/readline.h>
+
+/*
+ * Readline history functions - for some reason not all
+ * readline installs have history.h available, so we do
+ * this.
+ */
+#if !defined (PARAMS)
+#  if defined (__STDC__) || defined (__GNUC__) || defined (__cplusplus)
+#    define PARAMS(protos) protos
+#  else
+#    define PARAMS(protos) ()
+#  endif
+#endif /* PARAMS */
+
+extern void _rl_clear_screen PARAMS((void));
+
+#if !defined (_HISTORY_H_)
+extern void stifle_history   PARAMS((int));
+extern int  unstifle_history PARAMS((void));
+extern int  read_history     PARAMS((const char *));
+extern int  write_history    PARAMS((const char *));
+extern void add_history      PARAMS((const char *));
+#endif /*_HISTORY_H_ */
+
+#endif /* USE_READLINE */
+
 /*-- Prototypes --*/
 int   sqsh_readline_init  _ANSI_ARGS(( void ));
 int   sqsh_readline_exit  _ANSI_ARGS(( void ));
 char* sqsh_readline       _ANSI_ARGS(( char* )) ;
+int   sqsh_readline_load  _ANSI_ARGS(( void  )) ;
 int   sqsh_readline_read  _ANSI_ARGS(( char* )) ;
 int   sqsh_readline_add   _ANSI_ARGS(( char* )) ;
 int   sqsh_readline_clear _ANSI_ARGS(( void )) ;
-
 #endif /* sqsh_readline_h_included */
