@@ -29,10 +29,13 @@
 #include "sqsh_error.h"
 #include "sqsh_stdin.h"
 #include "var.h"
+#include "sqsh_global.h"
+#include "sqsh_fd.h"
+#include "sqsh_expand.h"
 
 /*-- Current Version --*/
 #if !defined(lint) && !defined(__LINT__)
-static char RCS_Id[] = "$Id: var_misc.c,v 1.1.1.1 2004/04/07 12:35:05 chunkm0nkey Exp $" ;
+static char RCS_Id[] = "$Id: var_misc.c,v 1.8 2014/03/12 14:40:43 mwesdorp Exp $" ;
 USE(RCS_Id)
 #endif /* !defined(lint) */
 
@@ -113,7 +116,7 @@ int var_set_esc( env, var_name, var_value )
 	/*
 	 * Allow them to set the string to a null value.
 	 */
-	if( *var_value == NULL || strcmp(*var_value,"NULL") == 0 ) {
+	if( var_value == NULL || *var_value == NULL || strcasecmp(*var_value,"NULL") == 0 ) {
 		*var_value = NULL ;
 		return True ;
 	}
@@ -124,7 +127,7 @@ int var_set_esc( env, var_name, var_value )
 		if( *src == '\\' ) {
 			++src ;
 			switch( *src ) {
-				case 'n' : 
+				case 'n' :
 					*dst++ = '\n' ;
 					break ;
 				case 'f' :
@@ -189,7 +192,7 @@ int var_set_notempty( env, var_name, var_value )
 		sqsh_set_error( SQSH_E_INVAL, "Invalid value" ) ;
 		return False ;
 	}
-		
+
 	return True ;
 }
 
@@ -218,7 +221,7 @@ int var_set_bool( env, var_name, var_value )
 		return True ;
 
 	/*-- Map True or Yes to 1 --*/
-	if( strcasecmp( *var_value, "True" ) == 0 || 
+	if( strcasecmp( *var_value, "True" ) == 0 ||
 		 strcasecmp( *var_value, "Yes" ) == 0 ||
 		 strcasecmp( *var_value, "On" ) == 0 ) {
 		*var_value = "1" ;
@@ -226,7 +229,7 @@ int var_set_bool( env, var_name, var_value )
 	}
 
 	/*-- Map False or No to 0 --*/
-	if( strcasecmp( *var_value, "False" ) == 0 || 
+	if( strcasecmp( *var_value, "False" ) == 0 ||
 		 strcasecmp( *var_value, "No" ) == 0 ||
 		 strcasecmp( *var_value, "Off" ) == 0 ) {
 		*var_value = "0" ;
@@ -244,8 +247,10 @@ int var_set_nullint( env, var_name, var_value )
 {
 	char *cptr ;
 
-	if( *var_value == NULL || strcmp(*var_value, "NULL") == 0 )
-		*var_value = "0" ;
+	if( var_value == NULL || *var_value == NULL || strcasecmp(*var_value, "NULL") == 0 ) {
+		*var_value = NULL ;
+		return True;
+	}
 
 	/*-- Skip whitespace --*/
 	for( cptr = *var_value; *cptr != '\0' && isspace( (int)*cptr ); ++cptr ) ;
@@ -269,7 +274,7 @@ int var_set_nullstr( env, var_name, var_value )
 	char     *var_name ;
 	char     **var_value ;
 {
-	if( *var_value == NULL || strcmp( *var_value, "NULL" ) == 0 )
+	if( var_value == NULL || *var_value == NULL || strcasecmp( *var_value, "NULL" ) == 0 )
 		*var_value = NULL ;
 
 	return True ;
@@ -286,7 +291,7 @@ int var_set_int( env, var_name, var_value )
 	char     **var_value ;
 {
 	/*-- Can't set it a NULL value --*/
-	if( var_value == NULL || *var_value == NULL ) {
+	if( var_value == NULL || *var_value == NULL || strcasecmp(*var_value, "NULL") == 0) {
 		sqsh_set_error( SQSH_E_INVAL, "Invalid integer expression" ) ;
 		return False ;
 	}
@@ -330,7 +335,7 @@ int var_set_add( env, var_name, var_value )
 
 	if( !isdigit((int)*value) )
 		n = value + 1 ;
-	else 
+	else
 		n = value ;
 
 	/*-- Check the digits --*/
@@ -411,3 +416,132 @@ int var_set_path_r( env, var_name, var_value )
 
 	return True;
 }
+
+/*
+ * var_set_lconv() - sqsh-2.3
+ *
+ * Validation function for setting variable localeconv (1/0, True/False, Yes/No)
+ * When set to true, initialize variable g_lconv to localeconv(), NULL otherwise.
+ * Returns True if validation succeed, False otherwise.
+ */
+int var_set_lconv( env, var_name, var_value )
+	env_t    *env ;
+	char     *var_name ;
+	char     **var_value ;
+{
+	/*
+	 * Perform simple validations.
+	 */
+	if( var_value == NULL || *var_value == NULL || **var_value == '\0' ) {
+		sqsh_set_error( SQSH_E_INVAL, "Invalid boolean value" ) ;
+		return False ;
+	}
+
+	/*-- Map True or Yes to 1 --*/
+	if( strcasecmp( *var_value, "True" ) == 0 ||
+	    strcasecmp( *var_value, "Yes" )  == 0 ||
+	    strcasecmp( *var_value, "On" )   == 0 ) {
+		*var_value = "1" ;
+	}
+	/*-- Map False or No to 0 --*/
+	else if( strcasecmp( *var_value, "False" ) == 0 ||
+		 strcasecmp( *var_value, "No" )    == 0 ||
+		 strcasecmp( *var_value, "Off" )   == 0 ) {
+		*var_value = "0" ;
+	}
+
+	if( strcmp( *var_value, "1" ) == 0 )
+	{
+#if defined(HAVE_LOCALE_H) && defined(HAVE_SETLOCALE) && defined(HAVE_LOCALECONV)
+		setlocale ( LC_ALL, "" );
+		g_lconv = localeconv();
+		return True;
+#else
+		g_lconv = NULL;
+		*var_value = "0" ;
+		sqsh_set_error( SQSH_E_INVAL, "setlocale() and/or localeconv() not available" ) ;
+		return False;
+#endif
+	}
+	else if( strcmp( *var_value, "0" ) == 0 )
+	{
+#if defined(HAVE_LOCALE_H) && defined(HAVE_SETLOCALE)
+		setlocale ( LC_ALL, "C" );
+#endif
+		g_lconv = NULL;
+		return True;
+	}
+
+	sqsh_set_error( SQSH_E_INVAL, "Invalid boolean value" ) ;
+	return False ;
+}
+
+/*
+ * sqsh-2.5 - New feature p2f (Print to File)
+ *            Assume var_name == "p2fname"
+ */
+int var_set_p2fname( env, var_name, var_value )
+	env_t    *env ;
+	char     *var_name ;
+	char     **var_value ;
+{
+	varbuf_t  *exp_buf = NULL;
+	FILE      *nfp     = NULL;
+	char      *exp_fn;
+
+
+	if ( var_name == NULL || strcmp(var_name, "p2fname") != 0 )
+       	{
+		sqsh_set_error( SQSH_E_INVAL, "var_set_p2fname: Unexpected variable name %s", var_name == NULL ? "NULL" : var_name ) ;
+		return False;
+	}
+
+	if ( var_value == NULL || *var_value == NULL || strcasecmp( *var_value, "NULL" ) == 0 )
+       	{
+		*var_value = NULL ;
+	}
+	else {
+		if ((exp_buf = varbuf_create( 512 )) == NULL)
+		{
+			fprintf( stderr, "sqsh: %s\n", sqsh_get_errstr() );
+			sqsh_set_error( SQSH_E_INVAL, "Unable to expand filename %s", *var_value ) ;
+			*var_value = NULL ;
+			return False;
+		}
+
+		if (sqsh_expand( *var_value, exp_buf, 0 ) != False)
+		{
+			exp_fn = varbuf_getstr( exp_buf );
+		}
+		else
+		{
+			sqsh_set_error( SQSH_E_INVAL, "Unable to expand filename %s", *var_value ) ;
+			*var_value = NULL ;
+			varbuf_destroy( exp_buf );
+			return False;
+		}
+
+		if( (nfp = fopen( exp_fn, "a" )) == NULL)
+	       	{
+			sqsh_set_error( SQSH_E_INVAL, "Unable to open file %s", exp_fn ) ;
+			*var_value = NULL ;
+			varbuf_destroy( exp_buf );
+			return False;
+		}
+		varbuf_destroy( exp_buf );
+	}
+
+	/*
+	 * If we come here, the fopen of the new file was successfull, or the p2fname variable is
+	 * set to NULL. Either way, we can close the original file pointer if it is open
+	 * and assign g_p2f_fp the new file pointer value (that still might be NULL).
+	 */
+	if (g_p2f_fp != NULL)
+       	{
+		fclose (g_p2f_fp);
+	}
+	g_p2f_fp = nfp;
+
+	return True ;
+}
+
