@@ -27,17 +27,12 @@
 #include "sqsh_config.h"
 #include "sqsh_env.h"
 #include "sqsh_error.h"
+#include "sqsh_readline.h"
 #include "var.h"
 
-#if defined(USE_READLINE)
-#include <readline/readline.h>
-extern void stifle_history();
-extern void unstifle_history();
-#endif
-
 /*-- Current Version --*/
 #if !defined(lint) && !defined(__LINT__)
-static char RCS_Id[] = "$Id: var_readline.c,v 1.1.1.1 2004/04/07 12:35:04 chunkm0nkey Exp $" ;
+static char RCS_Id[] = "$Id: var_readline.c,v 1.3 2013/12/24 13:23:19 mwesdorp Exp $" ;
 USE(RCS_Id)
 #endif /* !defined(lint) */
 
@@ -53,7 +48,7 @@ int var_set_rl_histsize( env, var_name, var_value )
 		return False;
 	}
 
-	stifle_value = atoi(*var_value);
+	stifle_value = (*var_value != NULL) ? atoi(*var_value) : 0;
 
 #if defined(USE_READLINE)
 	if (stifle_value <= 0)
